#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Name:     usb.py
# Purpose:  Module to list USB devices and get details under Linux and Windows
# Authors:  Sundar
# Licence:  This file is a part of multibootusb package. You can redistribute it or modify
# under the terms of GNU General Public License, v.2 or above

import sys
import platform
import os
import shutil
import collections
import ctypes
import subprocess
from . import config
from . import gen
if platform.system() == 'Linux':
    from . import udisks
    u = udisks.get_udisks(ver=None)
if platform.system() == 'Windows':
    import psutil
    import win32com.client
#     import wmi
    import pythoncom


def is_block(usb_disk):
    """
    Function to detect if the USB is block device
    :param usb_disk: USB disk path
    :return: True is devie is block device else False
    """
    import stat
    if platform.system() == 'Linux':
        if len(usb_disk) != 9:
            return False
    elif platform.system() == 'Windows':
        if len(usb_disk) != 2:
            return False
        else:
            return True
    try:
        mode = os.stat(usb_disk).st_mode
        gen.log(mode)
        gen.log(stat.S_ISBLK(mode))
    except:
        return False

    return stat.S_ISBLK(mode)


def disk_usage(mount_path):
    """
    Return disk usage statistics about the given path as a (total, used, free)
    namedtuple.  Values are expressed in bytes.
    """
    # Author: Giampaolo Rodola' <g.rodola [AT] gmail [DOT] com>
    # License: MIT
    _ntuple_diskusage = collections.namedtuple('usage', 'total used free')

    if platform.system() == "Linux":
        st = os.statvfs(mount_path)
        free = st.f_bavail * st.f_frsize
        total = st.f_blocks * st.f_frsize
        used = (st.f_blocks - st.f_bfree) * st.f_frsize

        return _ntuple_diskusage(total, used, free)

    elif platform.system() == "Windows":

        _, total, free = ctypes.c_ulonglong(), ctypes.c_ulonglong(), \
                         ctypes.c_ulonglong()
        if sys.version_info >= (3,) or isinstance(mount_path, unicode):
            fun = ctypes.windll.kernel32.GetDiskFreeSpaceExW
        else:
            fun = ctypes.windll.kernel32.GetDiskFreeSpaceExA
        ret = fun(mount_path, ctypes.byref(_), ctypes.byref(total), ctypes.byref(free))
        if ret == 0:
            raise ctypes.WinError()
        used = total.value - free.value

        return _ntuple_diskusage(total.value, used, free.value)
    else:
        raise NotImplementedError("Platform not supported.")


def list_devices(fixed=False):
    """
    List inserted USB devices.
    :return: USB devices as list.
    """
    devices = []
    if platform.system() == "Linux":
        try:
            # pyudev is good enough to detect USB devices on modern Linux machines.
            gen.log("Using pyudev for detecting USB drives...")
            try:
                import pyudev
            except Exception as e:
                gen.log('PyUdev is not installed on host system, using built-in.')
                from . import pyudev
            context = pyudev.Context()

            for device in context.list_devices(subsystem='block'):
                if fixed is True:
                    if device.get('DEVTYPE') in ['disk', 'partition'] and device.get('ID_PART_TABLE_TYPE'):
                        devices.append(str(device.get('DEVNAME')))
                        gen.log("\t" + device.get('DEVNAME'))
                else:
                    if device.get('ID_BUS') in ['usb'] and device.get('ID_PART_TABLE_TYPE'):
                        devices.append(str(device.get('DEVNAME')))
                        gen.log("\t" + device.get('DEVNAME'))

        except Exception as e:
            gen.log(e)
            import dbus
            bus = dbus.SystemBus()
            try:
                # You should come here only if your system does'nt have udev installed.
                # We will use udiskd2 for now.
                gen.log("Falling back to Udisks2..")
                ud_manager_obj = bus.get_object(
                    'org.freedesktop.UDisks2', '/org/freedesktop/UDisks2')
                ud_manager = dbus.Interface(
                    ud_manager_obj, 'org.freedesktop.DBus.ObjectManager')
                for k, v in ud_manager.GetManagedObjects().items():
                    drive_info = v.get('org.freedesktop.UDisks2.Block', {})
                    if fixed is True:
                        if drive_info.get('IdUsage') == "filesystem" and not drive_info.get('ReadOnly'):
                            device = drive_info.get('Device')
                            device = bytearray(device).replace(b'\x00', b'').decode('utf-8')
                            devices.append(device)
                    else:
                        if drive_info.get('IdUsage') == "filesystem" and not drive_info.get(
                                'HintSystem') and not drive_info.get('ReadOnly'):
                            device = drive_info.get('Device')
                            device = bytearray(device).replace(
                                b'\x00', b'').decode('utf-8')
                            devices.append(device)

            except Exception as e:
                gen.log(e, error=True)
                try:
                    # You must be using really old distro. Otherwise, the code
                    # should not reach here.
                    gen.log("Falling back to Udisks1...")
                    ud_manager_obj = bus.get_object(
                        "org.freedesktop.UDisks", "/org/freedesktop/UDisks")
                    ud_manager = dbus.Interface(
                        ud_manager_obj, 'org.freedesktop.UDisks')
                    for dev in ud_manager.EnumerateDevices():
                        device_obj = bus.get_object(
                            "org.freedesktop.UDisks", dev)
                        device_props = dbus.Interface(
                            device_obj, dbus.PROPERTIES_IFACE)
                        if device_props.Get('org.freedesktop.UDisks.Device',
                                            "DriveConnectionInterface") == "usb" and device_props.Get(
                                'org.freedesktop.UDisks.Device', "DeviceIsPartition"):
                            if device_props.Get('org.freedesktop.UDisks.Device', "DeviceIsMounted"):
                                device_file = device_props.Get(
                                    'org.freedesktop.UDisks.Device', "DeviceFile")
                                devices.append(device_file)
                except Exception as e:
                    gen.log(e, error=True)
                    gen.log("No USB device found...")

        devices.sort()

    elif platform.system() == "Windows":
        if fixed is True:
            for drive in psutil.disk_partitions():
                if 'cdrom' in drive.opts or drive.fstype == '':
                    # Skip cdrom drives or the disk with no filesystem
                    continue
                devices.append(drive[0][:-1])
        else:
            try:
                # Try new method using psutil. It should also detect USB 3.0 (but not tested by me)
                for drive in psutil.disk_partitions():
                    if 'cdrom' in drive.opts or drive.fstype == '':
                        # Skip cdrom drives or the disk with no filesystem
                        continue
                    if 'removable' in drive.opts:
                        devices.append(drive[0][:-1])
            except:
                # Revert back to old method if psutil fails (which is unlikely)
                oFS = win32com.client.Dispatch("Scripting.FileSystemObject")
                oDrives = oFS.Drives
                for drive in oDrives:
                    if drive.DriveType == 1 and drive.IsReady:
                        devices.append(drive)

    if devices:
        return devices
    else:
        gen.log("No USB device found...")
        return None


def details_udev(usb_disk_part):
    """
    Get details of USB partition using udev
    """
    assert usb_disk_part is not None
    assert platform.system() == "Linux"

    try:
        import pyudev
    except:
        from . import pyudev
#   Try with PyUdev to get the details of USB disks.
#   This is the easiest and reliable method to find USB details.
#   Also, it is a standalone package and no dependencies are required.
    context = pyudev.Context()
    try:
        device = pyudev.Device.from_device_file(context, usb_disk_part)
    except:
        gen.log("ERROR: Unknown disk/partition (%s)" % str(usb_disk_part))
        return None

    try:
        fdisk_cmd_out = subprocess.check_output('fdisk -l ' + usb_disk_part, shell=True)
    except subprocess.CalledProcessError:
        gen.log("ERROR: fdisk failed on disk/partition (%s)" % str(usb_disk_part))
        return None

    if b'Extended' in fdisk_cmd_out:
        mount_point = ''
        uuid = ''
        file_system = ''
        vendor = ''
        model = ''
        label = ''
        devtype = "extended partition"

    elif device.get('DEVTYPE') == "partition":
        uuid = device.get('ID_FS_UUID') or ""
        file_system = device.get('ID_FS_TYPE') or ""
        label = device.get('ID_FS_LABEL') or ""
        mount_point = u.mount(usb_disk_part) or ""
        mount_point = mount_point.replace('\\x20', ' ')
        vendor = device.get('ID_VENDOR') or ""
        model = device.get('ID_MODEL') or ""
        devtype = "partition"

    elif device.get('DEVTYPE') == "disk":
        mount_point = ""
        uuid = ""
        file_system = ""
        label = device.get('ID_FS_LABEL') or ""
        vendor = device.get('ID_VENDOR') or ""
        model = device.get('ID_MODEL') or ""
        devtype = "disk"

    if mount_point not in ["", "None"]:
        size_total = shutil.disk_usage(mount_point)[0]
        size_used = shutil.disk_usage(mount_point)[1]
        size_free = shutil.disk_usage(mount_point)[2]

    else:
        fdisk_cmd = 'fdisk -l ' + usb_disk_part + ' | grep "^Disk /" | sed -re "s/.*\s([0-9]+)\sbytes.*/\\1/"'
        size_total = subprocess.check_output(fdisk_cmd, shell=True).strip()
        size_used = ""
        size_free = ""
        mount_point = ""

    return {'uuid': uuid, 'file_system': file_system, 'label': label, 'mount_point': mount_point,
            'size_total': size_total, 'size_used': size_used, 'size_free': size_free,
            'vendor': vendor, 'model': model, 'devtype': devtype}


def details_udisks2(usb_disk_part):
    """
    Get details of USB disk detail.
    usb_disk_part: It is the partition of an USB removable disk.
    """
    import dbus
    bus = dbus.SystemBus()

    mount_point = ''
    uuid = ''
    file_system = ''
    vendor = ''
    model = ''
    label = ''
    devtype = "disk"

    bd = bus.get_object('org.freedesktop.UDisks2', '/org/freedesktop/UDisks2/block_devices%s'%usb_disk_part[4:])
    device = bd.Get('org.freedesktop.UDisks2.Block', 'Device', dbus_interface='org.freedesktop.DBus.Properties')
    device = bytearray(device).replace(b'\x00', b'').decode('utf-8')
    if device[-1].isdigit() is True:
        uuid = bd.Get('org.freedesktop.UDisks2.Block', 'IdUUID', dbus_interface='org.freedesktop.DBus.Properties')
        file_system = bd.Get('org.freedesktop.UDisks2.Block', 'IdType', dbus_interface='org.freedesktop.DBus.Properties')
        mount_point = bd.Get('org.freedesktop.UDisks2.Filesystem', 'MountPoints', dbus_interface='org.freedesktop.DBus.Properties')
        devtype = 'partition'
    else:
        devtype = 'disk'
        file_system = 'No_File_System'
    if mount_point:
        # mount_point = str(bytearray(mount_point[0]).decode('utf-8').replace(b'\x00', b''))
        mount_point = bytearray(mount_point[0]).replace(b'\x00', b'').decode('utf-8')
    else:
        try:
            mount_point = u.mount(usb_disk_part)
        except:
            mount_point = "No_Mount"
    try:
        label = bd.Get('org.freedesktop.UDisks2.Block', 'IdLabel', dbus_interface='org.freedesktop.DBus.Properties')
    except:
        label = "No_Label"
    usb_drive_id = (bd.Get('org.freedesktop.UDisks2.Block', 'Drive', dbus_interface='org.freedesktop.DBus.Properties'))
    bd1 = bus.get_object('org.freedesktop.UDisks2', usb_drive_id)
    try:
        vendor = bd1.Get('org.freedesktop.UDisks2.Drive', 'Vendor', dbus_interface='org.freedesktop.DBus.Properties')
    except:
        vendor = str('No_Vendor')
    try:
        model = bd1.Get('org.freedesktop.UDisks2.Drive', 'Model', dbus_interface='org.freedesktop.DBus.Properties')
    except:
        model = str('No_Model')
    if not mount_point == "No_Mount":
            size_total = shutil.disk_usage(mount_point)[0]
            size_used = shutil.disk_usage(mount_point)[1]
            size_free = shutil.disk_usage(mount_point)[2]
    else:
        size_total = str('No_Mount')
        size_used = str('No_Mount')
        size_free = str('No_Mount')

    return {'uuid': uuid, 'file_system': file_system, 'label': label, 'mount_point': mount_point,
            'size_total': size_total, 'size_used': size_used, 'size_free': size_free,
            'vendor': vendor, 'model': model, 'devtype': devtype}


def bytes2human(n):
    """
    Convert the size to human readable format
    Authored by 'Giampaolo Rodolà' and original link is:-
    http://code.activestate.com/recipes/577972-disk-usage/
    """
    try:
        n = int(n)
    except:
        return 'Unknown'
    symbols = ('K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y')
    prefix = {}
    for i, s in enumerate(symbols):
        prefix[s] = 1 << (i + 1) * 10
    for s in reversed(symbols):
        if n >= prefix[s]:
            value = float(n) / prefix[s]
            return '%.1f%s' % (value, s)
    return "%sB" % n


def gpt_device(dev_name):
    """
    Find if the device inserted is GPT or not. We will just change the variable parameter in config file for later use
    :param dev_name:
    :return: True if GPT else False
    """
    if platform.system() == 'Windows':
        diskpart_cmd = 'diskpart.exe /s ' + os.path.join('data', 'tools', 'gdisk', 'list-disk.txt')
        dev_no = get_physical_disk_number(dev_name)
        cmd_out = subprocess.check_output(diskpart_cmd)
        cmd_spt = cmd_out.split(b'\r')
        for line in cmd_spt:
            line = line.decode('utf-8')
            if 'Disk ' + dev_no in line:
                if '*' not in line.split()[-1]:
                    config.usb_gpt = False
                    gen.log('Device ' + dev_name + ' is a MBR disk...')
                    return False
                else:
                    config.usb_gpt = True
                    gen.log('Device ' + dev_name + ' is a GPT disk...')
                    return False
    if platform.system() == "Linux":
        if gen.has_digit(dev_name):
            _cmd_out = subprocess.check_output("parted  " + dev_name[:-1] + " print", shell=True)
        else:
            _cmd_out = subprocess.check_output("parted  " + dev_name + " print", shell=True)
        if b'msdos' in _cmd_out:
            config.usb_gpt = False
            gen.log('Device ' + dev_name + ' is a MBR disk...')
            return False
        elif b'gpt' in _cmd_out:
            config.usb_gpt = True
            gen.log('Device ' + dev_name + ' is a GPT disk...')
            return True


def win_disk_details(disk_drive):
    """
    Populate and get details of an USB disk under windows. Minimum required windows version is Vista.
    :param disk_drive: USB disk like 'G:'
    :return: See the details(usb_disk_part) function for return values.
    """
    pythoncom.CoInitialize()
    vendor = 'Not_Found'
    model = 'Not_Found'
    devtype = 'Not_Found'
    selected_usb_part = str(disk_drive)
    oFS = win32com.client.Dispatch("Scripting.FileSystemObject")
    d = oFS.GetDrive(oFS.GetDriveName(oFS.GetAbsolutePathName(selected_usb_part)))
    selected_usb_device = d.DriveLetter
    if d.DriveType == 1:
        devtype = "Removable Disk"
    elif d.DriveType == 2:
        devtype = "Fixed Disk"
    label = (d.VolumeName).strip()
    if not label.strip():
        label = "No_label"
    mount_point = selected_usb_device + ":\\"
    serno = "%X" % (int(d.SerialNumber) & 0xFFFFFFFF)
    uuid = serno[:4] + '-' + serno[4:]
    file_system = (d.FileSystem).strip()
    size_total = shutil.disk_usage(mount_point)[0]
    size_used = shutil.disk_usage(mount_point)[1]
    size_free = shutil.disk_usage(mount_point)[2]

    # The below code works only from vista and above. I have removed it as many people reported that the software
    # was not working under windows xp. Even then, it is significantly slow if 'All Drives' option is checked.
    # Removing the code doesn't affect the functionality as it is only used to find vendor id and model of the drive.
#     c = wmi.WMI()
#     for physical_disk in c.Win32_DiskDrive(InterfaceType="USB"):
#         for partition in physical_disk.associators("Win32_DiskDriveToDiskPartition"):
#             for logical_disk in partition.associators("Win32_LogicalDiskToPartition"):
#                 if logical_disk.Caption == disk_drive:
#                     vendor = (physical_disk.PNPDeviceID.split('&VEN_'))[1].split('&PROD_')[0]
#                     model = (physical_disk.PNPDeviceID.split('&PROD_'))[1].split('&REV_')[0]

    return {'uuid': uuid, 'file_system': file_system, 'label': label, 'mount_point': mount_point,
            'size_total': size_total, 'size_used': size_used, 'size_free': size_free,
            'vendor': vendor, 'model': model, 'devtype': devtype}


def details(usb_disk_part):
    """
    Populate and get details of an USB disk.
    :param usb_disk_part: USB disk. Example.. "/dev/sdb1" on Linux and "D:\" on Windows.
    :return:    label       == > returns name/label of an inserted USB device.
                mount_point == > returns mount path of an inserted USB device.
                uuid        == > returns uuid of an inserted USB device.
                file_system == > returns type of filesystem of an inserted USB device.
                device      == > returns device path of an inserted USB device.
                size_total  == > returns total size in MB/GB of an inserted USB device.
                size_free   == > returns free size in MB/GB of an inserted USB device.
                size_used   == > returns used size in MB/GB of an inserted USB device.
                vendor      == > returns the name of the manufacturer.
                model       == > returns the model name of the USB.
    """

    assert usb_disk_part is not None

    details = {}

    if platform.system() == 'Linux':
        try:
            details = details_udev(usb_disk_part)
        except:
            details = details_udisks2(usb_disk_part)
    elif platform.system() == 'Windows':
        details = win_disk_details(usb_disk_part)

    return details


def get_physical_disk_number(usb_disk):
    """
    Get the physical disk number as detected ny Windows.
    :param usb_disk: USB disk (Like F:)
    :return: Disk number.
    """
    import wmi
    c = wmi.WMI()
    for physical_disk in c.Win32_DiskDrive():
        for partition in physical_disk.associators("Win32_DiskDriveToDiskPartition"):
            for logical_disk in partition.associators("Win32_LogicalDiskToPartition"):
                if logical_disk.Caption == usb_disk:
                    # gen.log("Physical Device Number is " + partition.Caption[6:-14])
                    return str(partition.Caption[6:-14])


if __name__ == '__main__':
    usb_devices = list_devices()
    if usb_devices is not None:
        for dev in usb_devices:
            gen.log(details(dev))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Name:     imager.py
# Purpose:  Module to write ISO image to selected USB disk. Uses dd as backend.
# Authors:  Sundar
# Licence:  This file is a part of multibootusb package. You can redistribute it or modify
# under the terms of GNU General Public License, v.2 or above
# WARNING : Any boot-able USB made using this module will destroy data stored on USB disk.

import os
import subprocess
import collections
import platform
import signal
from PyQt5 import QtWidgets
from .gui.ui_multibootusb import Ui_MainWindow
from .gen import *
from . import iso
from . import config
from . import progressbar

if platform.system() == "Windows":
    import win32com.client


def dd_linux():
    import time
    _input = "if=" + config.image_path
    in_file_size = float(os.path.getsize(config.image_path))
    _output = "of=" + config.usb_disk
    os.system("umount " + config.usb_disk + "1")
    command = ['dd', _input, _output, "bs=1M", "oflag=sync"]
    log("Executing ==> " + " ".join(command))
    dd_process = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False)

    pbar = progressbar.ProgressBar(
            maxval=100,
            widgets=[
                ' ',
                progressbar.widgets.Bar(marker='=', left='[', right=']'),
                ' ',
                progressbar.widgets.Percentage()
            ]
    ).start()

    while dd_process.poll() is None:
        time.sleep(0.1)  # If this time delay is not given, the Popen does not execute the actual command
        dd_process.send_signal(signal.SIGUSR1)
        dd_process.stderr.flush()
        while True:
            time.sleep(0.1)
            out_error = dd_process.stderr.readline().decode()
            if out_error:
                if 'bytes' in out_error:
                    copied = int(out_error.split(' ', 1)[0])
                    config.imager_percentage = round((float(copied) / float(in_file_size) * 100))
                    pbar.update(config.imager_percentage)
                    break

    if dd_process.poll() is not None:
        log("\nExecuting ==> sync")
        os.sync()
        log("ISO has been written to USB disk...")
        return


def dd_win():

    windd = resource_path(os.path.join("data", "tools", "dd", "dd.exe"))
    if os.path.exists(resource_path(os.path.join("data", "tools", "dd", "dd.exe"))):
        log("dd exist")
    _input = "if=" + config.image_path
    in_file_size = float(os.path.getsize(config.image_path) / 1024 / 1024)
    _output = "of=\\\.\\" + config.usb_disk
    command = [windd, _input, _output, "bs=1M", "--progress"]
    log("Executing ==> " + " ".join(command))
    dd_process = subprocess.Popen(command, universal_newlines=True, stderr=subprocess.PIPE, stdout=subprocess.PIPE,
                                  shell=False)
    while dd_process.poll() is None:
        for line in iter(dd_process.stderr.readline, ''):
            line = line.strip()
            if 'error' in line.lower() or 'invalid' in line.lower():
                log("Error writing to disk...")
                break
            if line and line[-1] == 'M':
                copied = float(line.strip('M').replace(',', ''))
                config.imager_percentage = round((copied / float(in_file_size) * 100))

        log("ISO has been written to USB disk...")

        return


class Imager(QtWidgets.QMainWindow, Ui_MainWindow):
    """
    Raw write to USB disk using dd.
    """

    def __init__(self):
        QtWidgets.QMainWindow.__init__(self)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)


    def add_iso_gui_label_text(self):
        """
        Simple function to add text label to GUI widgets.
        :return:
        """
        log("Testing ISO...")
        self.ui.imager_bootable.setVisible(True)
        if iso.is_bootable(config.image_path) is True:
            self.ui.imager_bootable.setText("Bootable ISO: Yes")
            log("ISO is bootable.")
        else:
            self.ui.imager_bootable.setText("Bootable ISO: No")
            log("ISO is not bootable.")

        if os.path.exists(config.image_path):
            log("Path " + config.image_path + " exists...")
            self.iso_size = str(round(os.path.getsize(config.image_path) / 1024 / 1024))
            self.ui.imager_iso_size.setVisible(True)
            self.ui.imager_iso_size.setText("ISO Size: " + self.iso_size + " MB")
            log("ISO Size is " + self.iso_size + " MB")

    @staticmethod
    def imager_list_usb(partition=1):
        """
        Function to detect whole USB disk. It uses lsblk package on Linux.
        :param partition: What to return. By default partition is set.
        :return: USB disk/partition as list
        """
        disk = []
        if platform.system() == "Linux":
            output = subprocess.check_output("lsblk -i", shell=True)
            if partition != 1:
                for line in output.splitlines():
                    line = line.split()
                    if (line[2].strip()) == b'1' and (line[5].strip()) == b'disk':
                        disk.append(str("/dev/" + str(line[0].strip().decode())))
            elif partition == 1:
                for line in output.splitlines():
                    line = line.split()
                    if (line[2].strip()) == b'1' and line[5].strip() == b'part':
                        disk.append(str("/dev/" + str(line[0].strip()[2:])))
        else:
            oFS = win32com.client.Dispatch("Scripting.FileSystemObject")
            oDrives = oFS.Drives
            for drive in oDrives:
                if drive.DriveType == 1 and drive.IsReady:
                    disk.append(drive)
        return disk

    @staticmethod
    def imager_usb_detail(usb_disk, partition=1):
        """
        Function to detect details of USB disk using lsblk
        :param usb_disk: path to usb disk
        :param partition: by default partition is set (but yet to code for it)
        :return: details of size, type and model as tuples
        """
        _ntuple_diskusage = collections.namedtuple('usage', 'total_size usb_type model')

        if platform.system() == "Linux":
            output = subprocess.check_output("lsblk -ib " + usb_disk, shell=True)
            for line in output.splitlines():
                line = line.split()
                if partition != 1:
                    if line[2].strip() == b'1' and line[5].strip() == b'disk':
                        total_size = line[3]
                        if not total_size:
                            total_size = "Unknown"
                        usb_type = "Removable"
                        model = subprocess.check_output("lsblk -in -f -o MODEL " + usb_disk, shell=True).decode().strip()
                        if not model:
                            model = "Unknown"
        else:
            try:
                selected_usb_part = str(usb_disk)
                oFS = win32com.client.Dispatch("Scripting.FileSystemObject")
                d = oFS.GetDrive(oFS.GetDriveName(oFS.GetAbsolutePathName(selected_usb_part)))
#                 selected_usb_device = d.DriveLetter
                label = (d.VolumeName).strip()
                if not label.strip():
                    label = "No label."
                total_size = d.TotalSize
                usb_type = "Removable"
                model = label
            except:
                log("Error detecting USB details.")

        return _ntuple_diskusage(total_size, usb_type, model)

    def get_usb_size(self, usb_disk):
        """
        Function to detect USB disk space. Useful but not used in multibootusb as of now.
        :param usb_disk: USB disk like "/dev/sdb"
        :return: Size of the disk as integer
        """
        if platform.system() == "Linux":
            cat_output = subprocess.check_output("cat /proc/partitions | grep  " + usb_disk[5:], shell=True)
            usb_size = int(cat_output.split()[2]) * 1024
            # log(usb_size)
            return usb_size
        else:
            usb_size = self.usb.disk_usage(self.usb.get_usb(usb_disk).mount).total
            return usb_size



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Name:     syslinux.py
# Purpose:  Module to install syslinux and extlinux on selected USB disk.
# Authors:  Sundar
# Licence:  This file is a part of multibootusb package. You can redistribute it or modify
# under the terms of GNU General Public License, v.2 or above

import os
import subprocess
import platform
from .gen import *
from . import usb
from .iso import *
from . import config

extlinux_path = os.path.join(multibootusb_host_dir(), "syslinux", "bin", "extlinux4")
syslinux_path = os.path.join(multibootusb_host_dir(), "syslinux", "bin", "syslinux4")
extlinux_fs = ["ext2", "ext3", "ext4", "Btrfs"]
syslinux_fs = ["vfat", "ntfs", "FAT32", "NTFS"]
mbr_bin = resource_path(os.path.join("data", "tools", "mbr.bin"))
win_gdisk = resource_path(os.path.join('data', 'tools', 'gdisk', 'gdisk.exe'))


def gpt_part_table(usb_disk):
    """
    Check if selected USB contain GPT or MBR partition table
    :return: True if GPT else False
    """
    if platform.system() == "Linux":
        _cmd_out = subprocess.check_output("parted  " + usb_disk[:-1] + " print", shell=True)
        if b'msdos' in _cmd_out:
            return False
        elif b'gpt' in _cmd_out:
            return True
    elif platform.system() == 'Windows':
        win_usb_disk_no = str(usb.get_physical_disk_number(config.usb_disk))
        if config.usb_gpt is True:
            return True
        elif config.usb_gpt is False:
            return False


def get_mbr_bin_path(usb_disk):
    """
    Check if partition table type is mbr or gpr using parted command under Linux
    :param usb_disk: path to whole USB disk '/dev/sdb'
    :return: Path to mbr.bin for use
    """
    if config.usb_gpt is False:
        log('Using mbr.bin msdos mbr install.')
        return resource_path(os.path.join("data", "tools", "mbr.bin"))
    elif config.usb_gpt is True:
        log('Using gptmbr.bin for mbr install.')
        return resource_path(os.path.join("data", "tools", "gptmbr.bin"))

    return False


def set_boot_flag(usb_disk):
    if platform.system() == "Linux":
        log("\nChecking boot flag on " + usb_disk[:-1], '\n')
        cmd_out = subprocess.check_output("parted -m -s " + usb_disk[:-1] + " print", shell=True)
        if gpt_part_table(usb_disk) is False:
            if b'boot' in cmd_out:
                log("\nDisk " + usb_disk[:-1] + " already has boot flag.\n")
                return True
            else:
                log("\nExecuting ==>  parted " + usb_disk[:-1] + " set 1 boot on", '\n')
                if subprocess.call("parted " + usb_disk[:-1] + " set 1 boot on", shell=True) == 0:
                    log("\nBoot flag set to bootable " + usb_disk[:-1], '\n')
                    return True
                else:
                    log("\nUnable to set boot flag on  " + usb_disk[:-1], '\n')
                    return False
        elif gpt_part_table(usb_disk) is True:
            if b'legacy_boot' in cmd_out:
                log("\nGPT Disk " + usb_disk[:-1] + " already has legacy_boot flag.\n")
                return True
            else:
                log("\nExecuting ==>  parted " + usb_disk[:-1] + " set 1 legacy_boot on", '\n')
                if subprocess.call("parted " + usb_disk[:-1] + " set 1 legacy_boot on", shell=True) == 0:
                    log("\nBoot flag set to legacy_boot " + usb_disk[:-1], '\n')
                    return True
                else:
                    log("\nUnable to set legacy_boot flag on  " + usb_disk[:-1], '\n')
                    return False


def syslinux_default(usb_disk):
    """
    Install Syslinux of a selected drive
    :param usb_disk: '/dev/sdx' on linux and 'E:' on Windows
    :version: Default version is 4. Change it if you wish. But necessary files needs to be copied accordingly
    :return: Bootable USB disk :-)
    """
    usb_details = usb.details(usb_disk)
    usb_fs = usb_details['file_system']
    usb_mount = usb_details['mount_point']
    mbr_bin = get_mbr_bin_path(usb_disk)

    if platform.system() == 'Linux':
        mbr_install_cmd = 'dd bs=440 count=1 conv=notrunc if=' + mbr_bin + ' of=' + usb_disk[:-1]
    else:
        win_usb_disk_no = str(usb.get_physical_disk_number(config.usb_disk))
        _windd = resource_path(os.path.join("data", "tools", "dd", "dd.exe"))
        _input = "if=" + mbr_bin
        _output = 'of=\\\.\\physicaldrive' + win_usb_disk_no
        mbr_install_cmd = _windd + ' ' + _input + ' ' + _output + ' count=1'

    if usb_fs in extlinux_fs:
        extlinu_cmd = extlinux_path + ' --install ' + os.path.join(usb_mount, 'multibootusb')
        if os.access(extlinux_path, os.X_OK) is False:
            subprocess.call('chmod +x ' + extlinux_path, shell=True)
        log("\nExecuting ==> " + extlinu_cmd)
        config.status_text = 'Installing default extlinux version 4...'
        if subprocess.call(extlinu_cmd, shell=True) == 0:
            log("\nDefault Extlinux install is success...\n")
            config.status_text = 'Default extlinux install is success...'
            config.status_text = 'Installing mbr...'
            log('\nExecuting ==> ' + mbr_install_cmd)
            if subprocess.call(mbr_install_cmd, shell=True) == 0:
                config.status_text = 'mbr install is successful...'
                log("\nmbr install is success...\n")
                if set_boot_flag(usb_disk) is True:
                    return True

    elif usb_fs in syslinux_fs:

        if platform.system() == "Linux":
            syslinux_cmd = syslinux_path + ' -i -d multibootusb ' + usb_disk
            if os.access(syslinux_path, os.X_OK) is False:
                subprocess.call('chmod +x ' + syslinux_path, shell=True)
            log("\nExecuting ==> " + syslinux_cmd + "\n")
            config.status_text = 'Installing default syslinux version 4...'
            if subprocess.call(syslinux_cmd, shell=True) == 0:
                log("\nDefault syslinux install is success...\n")
                config.status_text = 'Default syslinux successfully installed...'
                log('\nExecuting ==> ' + mbr_install_cmd)
                if subprocess.call(mbr_install_cmd, shell=True) == 0:
                    config.status_text = 'mbr install is success...'
                    log("\nmbr install is success...\n")
                    if set_boot_flag(usb_disk) is True:
                        return True
                    else:
                        log("\nFailed to install default syslinux...\n")
                        config.status_text = 'Failed to install default syslinux...'
                        return False

        elif platform.system() == "Windows":
            syslinux = resource_path(os.path.join(multibootusb_host_dir(), "syslinux", "bin", "syslinux4.exe"))
            config.status_text = 'Installing default syslinux version 4...'
            # syslinux_cmd = syslinux + ' -maf -d multibootusb ' + usb_disk
            if config.usb_gpt is False:
                syslinux_cmd = syslinux + ' -maf -d multibootusb ' + usb_disk
            else:
                syslinux_cmd = syslinux + ' -af -d multibootusb ' + usb_disk
            log('Executing ==> ' + syslinux_cmd)
            '''
            if gpt_part_table(config.usb_disk) is False:
                syslinux_cmd = syslinux + ' -maf -d multibootusb ' + usb_disk
            else:
                syslinux_cmd = syslinux + ' -af -d multibootusb ' + usb_disk
            '''
            if subprocess.call(syslinux_cmd, shell=True) == 0:
                config.status_text = 'Default syslinux successfully installed...'
                log("\nDefault syslinux install is success...\n")
                # We will need to flash gptmbr.bin only for GPT disk. As of version 8.9.0 this corrupts the gpt disk.
                # Therefore not included for BIOS booting. GPT disk may work on UEFI system. 
                # if gpt_part_table(config.usb_disk) is True:
                '''
                if config.usb_gpt is False:
                    log('\nExecuting ==> ' + mbr_install_cmd)
                    if subprocess.call(mbr_install_cmd, shell=True) == 0:
                        log("\nmbr install is success...\n")
                        return True
                else:
                    log('Disk uses GPT and mbr install is not required...')
                '''

            else:
                log("\nFailed to install default syslinux...\n")
                config.status_text = 'Failed to install default syslinux...'
                return False


def syslinux_distro_dir(usb_disk, iso_link, distro):
    """
    Install syslinux/extlinux on distro specific isolinux directory.
    :param usb_disk: '/dev/sdx' on linux and 'E:' on Windows
    :param iso_link: Path to ISO file
    :return:
    """
    usb_details = usb.details(usb_disk)
    usb_fs = usb_details['file_system']
    usb_mount = usb_details['mount_point']
    isolinux_bin_dir(iso_link)
    if isolinux_bin_exist(iso_link) is False:
        log('Distro does not use isolinux for booting ISO.')
    else:
        # iso_cfg_ext_dir = os.path.join(multibootusb_host_dir(), "iso_cfg_ext_dir")
        _iso_cfg_ext_dir = iso_cfg_ext_dir()
        isolinux_path = os.path.join(_iso_cfg_ext_dir, isolinux_bin_path(iso_link))
        iso_linux_bin_dir = isolinux_bin_dir(iso_link)
        config.syslinux_version = isolinux_version(isolinux_path)
        if int(config.syslinux_version) < 3:
            log('Distro uses really old isolinux. Installing version 3 instead of 2.')
            config.syslinux_version = '3'

        if distro in ["generic"]:
            install_dir = usb_mount
            distro_syslinux_install_dir = os.path.join(usb_mount, iso_linux_bin_dir.strip("/")).replace(usb_mount, "")
            distro_sys_install_bs = os.path.join(install_dir, iso_linux_bin_dir.strip("/"), distro + '.bs')
        else:
            install_dir = os.path.join(usb_mount, "multibootusb", iso_basename(iso_link))
            distro_syslinux_install_dir = os.path.join(install_dir, iso_linux_bin_dir.strip("/")).replace(usb_mount, "")
            distro_sys_install_bs = os.path.join(install_dir, iso_linux_bin_dir.strip("/"), distro + '.bs')
#             log(distro_sys_install_bs)
#             log(distro_syslinux_install_dir)

        if usb_fs in syslinux_fs:
            if config.syslinux_version == str(3):
                if distro == "generic" and iso_linux_bin_dir == "/":
                    option = ""
                else:
                    option = " -d "
            else:
                if distro == "generic" and iso_linux_bin_dir == "/":
                    option = " -i "
                else:
                    option = " -i -d "

            if platform.system() == "Linux":
                syslinux_path = os.path.join(multibootusb_host_dir(), "syslinux", "bin", "syslinux") + config.syslinux_version
                if os.access(syslinux_path, os.X_OK) is False:
                    subprocess.call('chmod +x ' + syslinux_path, shell=True)
                sys_cmd = syslinux_path + option + quote(distro_syslinux_install_dir) + ' ' + usb_disk
                dd_cmd = 'dd if=' + usb_disk + ' ' + 'of=' + quote(distro_sys_install_bs) + ' count=1'
                log("Executing ==> " + sys_cmd)
                config.status_text = 'Installing distro specific syslinux...'
                if subprocess.call(sys_cmd, shell=True) == 0:
                    config.status_text = 'Syslinux install on distro directory is successful...'
                    log("\nSyslinux install on distro directory is successful...\n")
                    log('Executing ==> ' + dd_cmd + '\n')
                    config.status_text = 'Copying boot sector...'
                    if subprocess.call(dd_cmd, shell=True) == 0:
                        config.status_text = 'Bootsector copy is successful...'
                        log("\nBootsector copy is successful...\n")
                    else:
                        config.status_text = 'Failed to copy boot sector...'
                        log("\nFailed to copy boot sector...\n")
                else:
                    config.status_text = 'Failed to install syslinux on distro directory...'
                    log("\nFailed to install syslinux on distro directory...\n")

            elif platform.system() == "Windows":
                syslinux_path = resource_path(os.path.join(multibootusb_host_dir(), "syslinux", "bin")) + \
                                "\syslinux" + config.syslinux_version + ".exe"
                distro_syslinux_install_dir = "/" + distro_syslinux_install_dir.replace("\\", "/")
                distro_sys_install_bs = distro_sys_install_bs.replace("/", "\\")
                sys_cmd = syslinux_path + option + distro_syslinux_install_dir + ' ' + usb_disk + ' ' + \
                          distro_sys_install_bs
                log("\nExecuting ==> " + sys_cmd + '\n')
                config.status_text = 'Installing distro specific syslinux...'
                if subprocess.call(sys_cmd, shell=True) == 0:
                    config.status_text = 'Syslinux install on distro directory is successful...'
                    log("\nSyslinux install was successful on distro directory...\n")
                else:
                    config.status_text = 'Failed to install syslinux on distro directory...'
                    log("\nFailed to install syslinux on distro directory...\n")
        elif usb_fs in extlinux_fs:
            if platform.system() == "Linux":
                distro_syslinux_install_dir = os.path.join(install_dir, iso_linux_bin_dir.strip("/"))
                syslinux_path = os.path.join(multibootusb_host_dir(), "syslinux", "bin", "extlinux") + config.syslinux_version
                ext_cmd = syslinux_path + " --install " + distro_syslinux_install_dir
                dd_cmd = 'dd if=' + usb_disk + ' ' + 'of=' + quote(distro_sys_install_bs) + ' count=1'
                if os.access(syslinux_path, os.X_OK) is False:
                    subprocess.call('chmod +x ' + syslinux_path, shell=True)
                log("Executing ==> " + ext_cmd)
                if subprocess.call(ext_cmd, shell=True) == 0:
                    log("\nSyslinux install on distro directory is successful...\n")
                    log('Executing ==> ' + dd_cmd, '\n')
                    if subprocess.call(dd_cmd, shell=True) == 0:
                        log("\nBootsector copy is successful...\n")
                    else:
                        log("\nFailed to install syslinux on distro directory...\n")


def replace_grub_binary():
    """
    This function checks if correct binary is installed on grub and EFI directory.
    If mismatch is found between partition table and binary, replace it correct one.
    Default binaries will work for msdos partition table and therefore need not be replaced.
    :return: 
    """
    grub_efi_bin_path = os.path.join(config.usb_mount, 'EFI', 'BOOT', 'bootx64.efi')
    grub_efi_bin_gpt_path = os.path.join(config.usb_mount, 'EFI', 'BOOT', 'bootx64-gpt.efi')
    grub_efi_bin_msdos_path = os.path.join(config.usb_mount, 'EFI', 'BOOT', 'bootx64-msdos.efi')
    core_img_bin_path = os.path.join(config.usb_mount, 'multibootusb', 'grub', 'core.img')
    core_img_bin_msdos_path = os.path.join(config.usb_mount, 'multibootusb', 'grub', 'core-msdos.img')
    core_img_bin_gpt_path = os.path.join(config.usb_mount, 'multibootusb', 'grub', 'core-gpt.img')

    if platform.system() in ['Linux', 'Windows']:
        if gpt_part_table(config.usb_disk) is True:
            log('Replacing efi binary with gpt compatible one...')
            try:
                shutil.copy(grub_efi_bin_gpt_path, grub_efi_bin_path)
            except Exception as e:
                log(e)
                log('Failed to replace efi binary...')
            log('Replacing core.img binary with gpt compatible one...')
            try:
                shutil.copy(core_img_bin_gpt_path, core_img_bin_path)
            except Exception as e:
                log(e)
                log('Failed to replace efi binary...')
        else:
            log('Replacing efi binary with msdos compatible one...')
            try:
                # shutil.copy(core_img_bin_gpt_path, core_img_bin_path)
                shutil.copy(grub_efi_bin_msdos_path, grub_efi_bin_path)
            except Exception as e:
                log(e)
                log('Failed to replace efi binary...')
            try:
                log('Replacing core.img with msdos compatible one...')
                shutil.copy(core_img_bin_msdos_path, core_img_bin_path)
            except Exception as e:
                log(e)
                log('Failed to replace core.img binary...')


if __name__ == '__main__':
    if os.geteuid() != 0:
        log('Please running this script with sudo/root/admin privilage.')
        exit(1)
    else:
        syslinux_distro_dir('/dev/sdb1', '../../../DISTROS/2016/debian-live-8.3.0-amd64-lxde-desktop.iso', 'debian')
        syslinux_default('/dev/sdb1')



# -*- coding: utf-8 -*-
# Copyright (C) 2010, 2011, 2012 Sebastian Wiesner <lunaryorn@gmail.com>

# This library is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation; either version 2.1 of the License, or (at your
# option) any later version.

# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
# for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation,
# Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA


"""
    pyudev
    ======

    A binding to libudev.

    The :class:`Context` provides the connection to the udev device database
    and enumerates devices.  Individual devices are represented by the
    :class:`Device` class.

    Device monitoring is provided by :class:`Monitor` and
    :class:`MonitorObserver`.  With :mod:`pyudev.pyqt4`, :mod:`pyudev.pyside`,
    :mod:`pyudev.glib` and :mod:`pyudev.wx` device monitoring can be integrated
    into the event loop of various GUI toolkits.

    .. moduleauthor::  Sebastian Wiesner  <lunaryorn@gmail.com>
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from ._errors import DeviceNotFoundAtPathError
from ._errors import DeviceNotFoundByFileError
from ._errors import DeviceNotFoundByNameError
from ._errors import DeviceNotFoundByNumberError
from ._errors import DeviceNotFoundError
from ._errors import DeviceNotFoundInEnvironmentError

from .device import Attributes
from .device import Device
from .device import Devices
from .device import Tags

from .discover import DeviceFileHypothesis
from .discover import DeviceNameHypothesis
from .discover import DeviceNumberHypothesis
from .discover import DevicePathHypothesis
from .discover import Discovery

from .core import Context
from .core import Enumerator

from .monitor import Monitor
from .monitor import MonitorObserver

from .version import __version__
from .version import __version_info__

from ._util import udev_version



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Name:     install.py
# Purpose:  This module contain functions to install ISO files to USB disk non destructively.
# Authors:  Sundar
# Licence:  This file is a part of multibootusb package. You can redistribute it or modify
# under the terms of GNU General Public License, v.2 or above

import os
import shutil
import platform
import threading
import subprocess
import time
from .usb import *
from .gen import *
# from .iso import *
from . import iso
from scripts.update_cfg_file import *
from . import config
from . import persistence


def install_distro():
    """
    Install selected ISO to USB disk.
    :return:
    """
    usb_mount = config.usb_mount
    install_dir = os.path.join(config.usb_mount, "multibootusb", iso_basename(config.image_path))
    _iso_file_list = iso.iso_file_list(config.image_path)

    if not os.path.exists(os.path.join(usb_mount, "multibootusb")):
        log("Copying multibootusb directory to " + usb_mount)
        shutil.copytree(resource_path(os.path.join("data", "tools", "multibootusb")),
                        os.path.join(config.usb_mount, "multibootusb"))

    if not os.path.exists(install_dir):
        os.makedirs(install_dir)
        with open(os.path.join(install_dir, "multibootusb.cfg"), "w") as f:
            f.write(config.distro)
        with open(os.path.join(install_dir, "iso_file_list.cfg"), 'w') as f:
            for file_path in _iso_file_list:
                f.write(file_path + "\n")
    log("Installing " + iso_name(config.image_path) + " on " + install_dir)

    if config.distro == "opensuse":
        iso.iso_extract_file(config.image_path, install_dir, 'boot')
        config.status_text = "Copying ISO..."
        if platform.system() == "Windows":
            subprocess.call(["xcopy", config.image_path, usb_mount], shell=True)  # Have to use xcopy as python file copy is dead slow.
        elif platform.system() == "Linux":
            log("Copying " + config.image_path + " to " + usb_mount)
            shutil.copy(config.image_path, usb_mount)
    elif config.distro == "Windows" or config.distro == 'pc-unlocker'\
            or config.distro == 'pc-tool' or config.distro == 'grub2only':
        log("Extracting iso to " + usb_mount)
        iso_extract_full(config.image_path, usb_mount)
    elif config.distro == "trinity-rescue":
        iso_extract_full(config.image_path, install_dir)
        if os.path.exists(os.path.join(usb_mount, 'trk3')):
            shutil.rmtree(os.path.join(usb_mount, 'trk3'))
        shutil.move(os.path.join(install_dir, 'trk3'), os.path.join(usb_mount))
    elif config.distro == "ipfire":
        iso.iso_extract_file(config.image_path, usb_mount, '*.tlz')
        iso.iso_extract_file(config.image_path, usb_mount, 'distro.img')
        iso.iso_extract_file(config.image_path, install_dir, 'boot')
    elif config.distro == "zenwalk":
        config.status_text = "Copying ISO..."
        iso.iso_extract_file(config.image_path, install_dir, "kernel")
        copy_iso(config.image_path, install_dir)
    elif config.distro == "salix-live":
        # iso.iso_extract_file(config.image_path, install_dir, "boot")
        iso.iso_extract_file(config.image_path, install_dir, '*syslinux')
        iso.iso_extract_file(config.image_path, install_dir, '*menus')
        iso.iso_extract_file(config.image_path, install_dir, '*vmlinuz')
        iso.iso_extract_file(config.image_path, install_dir, '*initrd*')
        iso.iso_extract_file(config.image_path, usb_mount, '*modules')
        iso.iso_extract_file(config.image_path, usb_mount, '*packages')
        iso.iso_extract_file(config.image_path, usb_mount, '*optional')
        iso.iso_extract_file(config.image_path, usb_mount, '*liveboot')
        #iso.iso_extract_full(config.image_path, usb_mount)
        config.status_text = "Copying ISO..."
        copy_iso(config.image_path, install_dir)
    elif config.distro == "rising-av":
        iso.iso_extract_file(config.image_path, install_dir, '*boot')
        iso.iso_extract_file(config.image_path, usb_mount, '*rising')
    elif config.distro == 'sgrubd2':
        copy_iso(config.image_path, install_dir)
    elif config.distro == 'alt-linux':
        iso.iso_extract_file(config.image_path, install_dir, '-xr!*rescue')
        iso.iso_extract_file(config.image_path, config.usb_mount, 'rescue')
    elif config.distro == "generic":
        #with open(os.path.join(install_dir, "generic.cfg"), "w") as f:
        #    f.write(os.path.join(isolinux_bin_dir(config.image_path), "generic") + ".bs")
        iso_extract_full(config.image_path, usb_mount)
    elif config.distro == 'grub4dos':
        iso_extract_full(config.image_path, usb_mount)
    elif config.distro == 'ReactOS':
        iso_extract_full(config.image_path, usb_mount)
    elif config.distro == 'grub4dos_iso' or config.distro == 'raw_iso' or config.distro == 'memdisk_iso' or \
                    config.distro == 'memdisk_img':
        copy_iso(config.image_path, install_dir)
    elif config.distro == 'Avira-RS':
        iso_extract_full(config.image_path, install_dir)
        # we want following directories on root of the USB drive. Ensure the previous directories are removed before moving.
        if os.path.exists(os.path.join(usb_mount, 'antivir')):
            shutil.rmtree(os.path.join(usb_mount, 'antivir'))
            shutil.move(os.path.join(install_dir, 'antivir'), os.path.join(usb_mount))
        if os.path.exists(os.path.join(usb_mount, 'avupdate')):
            shutil.rmtree(os.path.join(usb_mount, 'avupdate'))
            shutil.move(os.path.join(install_dir, 'avupdate'), os.path.join(usb_mount))
        if os.path.exists(os.path.join(usb_mount, 'system')):
            shutil.rmtree(os.path.join(usb_mount, 'system'))
            shutil.move(os.path.join(install_dir, 'system'), os.path.join(usb_mount))
    elif config.distro == 'alpine':
        iso_extract_full(config.image_path, install_dir)
        if os.path.exists(os.path.join(usb_mount, 'apks')):
            shutil.rmtree(os.path.join(usb_mount, 'apks'))
        shutil.move(os.path.join(install_dir, 'apks'), os.path.join(usb_mount))
    elif config.distro == 'insert':
        iso_extract_full(config.image_path, install_dir)
        if os.path.exists(os.path.join(usb_mount, 'INSERT')):
            shutil.rmtree(os.path.join(usb_mount, 'INSERT'))
        shutil.move(os.path.join(install_dir, 'INSERT'), os.path.join(usb_mount))
    else:
        iso.iso_extract_full(config.image_path, install_dir)

    if platform.system() == 'Linux':
        log('ISO extracted successfully. Sync is in progress...')
        os.sync()

    if config.persistence != 0:
        log('Creating persistence...')
        config.status_text = 'Creating persistence...'
        persistence.create_persistence()

    install_patch()


def copy_iso(src, dst):
    """
    A simple wrapper for copying larger files. This is necessary as
    shutil copy files is much slower under Windows platform
    :param src: Path to source file
    :param dst: Destination directory
    :return:
    """
    if platform.system() == "Windows":
        subprocess.call("xcopy " + src + " " + dst, shell=True)
    elif platform.system() == "Linux":
        shutil.copy(src, dst)


def install_progress():
    """
    Function to calculate progress percentage of install.
    :return:
    """
    from . import progressbar

    usb_details = details(config.usb_disk)
    config.usb_mount = usb_details['mount_point']
    usb_size_used = usb_details['size_used']
    thrd = threading.Thread(target=install_distro, name="install_progress")
    # thrd.daemon()
    # install_size = usb_size_used / 1024
#     install_size = iso_size(config.image_path) / 1024
    final_size = (usb_size_used + iso_size(config.image_path)) + config.persistence
    thrd.start()
    pbar = progressbar.ProgressBar(maxval=100).start()  # bar = progressbar.ProgressBar(redirect_stdout=True)
    while thrd.is_alive():
        current_size = shutil.disk_usage(usb_details['mount_point'])[1]
        percentage = int((current_size / final_size) * 100)
        if percentage > 100:
            percentage = 100
        config.percentage = percentage
        pbar.update(percentage)
        time.sleep(0.1)


def install_patch():
    """
    Function to certain distros which uses makeboot.sh script for making bootable usb disk.
    This is required to make sure that same version (32/64 bit) of modules present is the isolinux directory
    :return:
    """
    if config.distro == 'debian':
        if platform.system() == 'Linux':  # Need to syn under Linux. Otherwise, USB disk becomes random read only.
            os.sync()
        iso_cfg_ext_dir = os.path.join(multibootusb_host_dir(), "iso_cfg_ext_dir")
        isolinux_path = os.path.join(iso_cfg_ext_dir, isolinux_bin_path(config.image_path))
#         iso_linux_bin_dir = isolinux_bin_dir(config.image_path)
        config.syslinux_version = isolinux_version(isolinux_path)
        iso_file_list = iso.iso_file_list(config.image_path)
        os.path.join(config.usb_mount, "multibootusb", iso_basename(config.image_path), isolinux_bin_dir(config.image_path))
        if any("makeboot.sh" in s.lower() for s in iso_file_list):
            for module in os.listdir(os.path.join(config.usb_mount, "multibootusb", iso_basename(config.image_path),
                                                  isolinux_bin_dir(config.image_path))):
                if module.endswith(".c32"):
                    if os.path.exists(os.path.join(config.usb_mount, "multibootusb", iso_basename(config.image_path),
                                                   isolinux_bin_dir(config.image_path), module)):
                        try:
                            os.remove(os.path.join(config.usb_mount, "multibootusb",
                                                   iso_basename(config.image_path), isolinux_bin_dir(config.image_path), module))
                            log("Copying " +  module)
                            log((resource_path(
                                os.path.join(multibootusb_host_dir(), "syslinux", "modules", config.syslinux_version, module)),
                                        os.path.join(config.usb_mount, "multibootusb", iso_basename(config.image_path),
                                                     isolinux_bin_dir(config.image_path), module)))
                            shutil.copy(resource_path(
                                os.path.join(multibootusb_host_dir(), "syslinux", "modules", config.syslinux_version, module)),
                                        os.path.join(config.usb_mount, "multibootusb", iso_basename(config.image_path),
                                                     isolinux_bin_dir(config.image_path), module))
                        except Exception as err:
                            log(err)
                            log("Could not copy " + module)
        else:
            log('Patch not required...')


if __name__ == '__main__':
    config.image_path = '../../../DISTROS/2016/slitaz-4.0.iso'
    install_distro()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Name:     persistence.py
# Purpose:  Module to deal with persistence of a selected distro.
# Authors:  Sundar
# Licence:  This file is a part of multibootusb package. You can redistribute it or modify
# under the terms of GNU General Public License, v.2 or above

import os
import platform
import tarfile
import subprocess
from . import iso
from . import gen
from . import config

def max_disk_persistence(usb_disk):
    """
    Detect max persistence value for filesystem on usb_disk
    :param usb_disk: Disk filesystem to check
    :return: Max persistence supported (bytes)
    """
    assert usb_disk is not None

    fat_max_size = (4096 * 1024 * 1024)
    usb_details = config.usb_details
    config.usb_uuid = usb_details['uuid']
    config.usb_label = usb_details['label']

    if usb_details['file_system'] in ['vfat', 'FAT32'] and usb_details['size_free'] > fat_max_size:
        _max_size = fat_max_size
    else:
        _max_size = usb_details['size_free']

    return _max_size

def persistence_distro(distro, iso_link):
    """
    Function to detect if distro can have persistence option.
    :param distro: Detected distro name.
    :return: Distro name as string or None otherwise.
    """
    assert distro is not None
    assert iso_link is not None

#     iso_size = iso.iso_size(iso_link)

    if distro in ["ubuntu", "debian", "debian-install", "fedora"]:
        gen.log("Persistence option is available.")
        return distro
    else:
        return None


def create_persistence():
    if config.distro == "ubuntu":
        fs_name = 'casper-rw'
    elif config.distro == 'debian' or config.distro == "debian-install":
        fs_name = 'live-rw'
    elif config.distro == 'fedora':
        fs_name = 'overlay-' + config.usb_label + '-' + config.usb_uuid

    persistence = config.persistence / 1024 / 1024

    if platform.system() == 'Linux':
        mkfs = 'mkfs.ext3'
        dd = 'dd'
        persistence_mkfs_cmd = mkfs + ' -F ' + gen.quote(os.path.join(config.usb_mount, 'multibootusb',
                                                            iso.iso_basename(config.image_path),
                                                            fs_name))
    elif platform.system() == 'Windows':
        mkfs = gen.quote(gen.resource_path(os.path.join("data", "tools", "mkfs", "mke2fs.exe")))
        dd = gen.quote(gen.resource_path(os.path.join("data", "tools", "dd", "dd.exe")))
        persistence_mkfs_cmd = 'echo y|' + mkfs + ' -b 1024 -L ' + fs_name + ' ' + gen.quote(os.path.join(config.usb_mount, 'multibootusb',
                                                            iso.iso_basename(config.image_path), fs_name))

    if config.distro == 'fedora':
        persistence_dd_cmd = dd + ' if=/dev/zero ' \
                                  'of=' + gen.quote(os.path.join(config.usb_mount, 'multibootusb',
                                                       iso.iso_basename(config.image_path), 'LiveOS', fs_name)) + \
                             ' bs=1M count=' + str(int(persistence))
    else:
        persistence_dd_cmd = dd + ' if=/dev/zero of=' + gen.quote(os.path.join(config.usb_mount, 'multibootusb',
                                                   iso.iso_basename(config.image_path), fs_name)) +\
                                ' bs=1M count=' + str(int(persistence))

    gen.log('Executing ==>' + persistence_dd_cmd)
    config.status_text = 'Creating persistence file...'

    if subprocess.call(persistence_dd_cmd, shell=True) == 0:
        gen.log("\nSuccessfully created persistence file...\n")

    if config.distro != 'fedora':
        gen.log('Applying filesystem to persistence file...')
        config.status_text = 'Applying filesystem to persistence file. Please wait...'
        gen.log('Executing ==> ' + persistence_mkfs_cmd)
        config.status_text = 'Applying filesystem to persistence file...'
        if subprocess.call(persistence_mkfs_cmd, shell=True) == 0:
            gen.log("\nSuccessfully applied filesystem...\n")


def extract_file(file_path, install_dir):
    """
    Function to extract persistence files to distro install directory.
    :param file_path: Path to persistence file.
    :param install_dir: Path to distro install directory.
    :return:
    """
    tar = tarfile.open(file_path, "r:bz2")
    tar.extractall(install_dir)
    tar.close()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# progressbar  - Text progress bar library for Python.
# Copyright (c) 2005 Nilton Volpato
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""Text progress bar library for Python.

A text progress bar is typically used to display the progress of a long
running operation, providing a visual cue that processing is underway.

The ProgressBar class manages the current progress, and the format of the line
is given by a number of widgets. A widget is an object that may display
differently depending on the state of the progress bar. There are three types
of widgets:
 - a string, which always shows itself

 - a ProgressBarWidget, which may return a different value every time its
   update method is called

 - a ProgressBarWidgetHFill, which is like ProgressBarWidget, except it
   expands to fill the remaining width of the line.

The progressbar module is very easy to use, yet very powerful. It will also
automatically enable features like auto-resizing when the system supports it.
"""

__author__ = 'Nilton Volpato'
__author_email__ = 'first-name dot last-name @ gmail.com'
__date__ = '2011-05-14'
__version__ = '2.3'

from .compat import *
from .widgets import *
from .progressbar import *



#!/usr/bin/env python3
# -*- coding: utf-8 -*
# Name:     gen.py
# Purpose:  This 'general' module contain many functions required to be called at many places
# Authors:  Sundar
# Licence:  This file is a part of multibootusb package. You can redistribute it or modify
# under the terms of GNU General Public License, v.2 or above

import logging
import sys
import os
import platform
import shutil
import string
import zipfile
import tempfile
import re
import ctypes


def scripts_dir_path():
    return os.path.dirname(os.path.realpath(__file__))


def log(message, info=True, error=False, debug=False):
    """
    Dirty function to log messages to file and also print on screen.
    :param message:
    :param info:
    :param error:
    :param debug:
    :return:
    """
    # LOG_FILE_PATH = os.path.join(multibootusb_host_dir(), 'multibootusb.log')
    LOG_FILE_PATH = mbusb_log_file()
    if os.path.exists(LOG_FILE_PATH):
        log_file_size = os.path.getsize(LOG_FILE_PATH) / (1024.0 * 1024.0)
        if log_file_size > 1:
            print('Removing log file as it crosses beyond 1mb')
            os.remove(LOG_FILE_PATH)
    logging.basicConfig(filename=LOG_FILE_PATH,
                        filemode='a',
                        format='%(asctime)s.%(msecs)03d %(name)s %(levelname)s %(message)s',
                        datefmt='%H:%M:%S',
                        level=logging.DEBUG)
    print(message)

    # remove ANSI color codes from logs
    # message_clean = re.compile(r'\x1b[^m]*m').sub('', message)

    if info is True:
        logging.info(message)
    elif error is not False:
        logging.error(message)
    elif debug is not False:
        logging.debug(message)



def resource_path(relativePath):
    """
    Function to detect the correct path of file when working with sourcecode/install or binary.
    :param relativePath: Path to file/data.
    :return: Modified path to file/data.
    """

    try:
        basePath = sys._MEIPASS  # Try if we are running as standalone executable
        # log('Running stand alone executable.')
    except:
        basePath = '/usr/share/multibootusb'  # Check if we run in installed environment
        #if os.path.exists('/usr/share/multibootusb'):
            #log('Running from installed machine.')
        if not os.path.exists(basePath):
            #basePath = os.path.abspath(".")
            basePath = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))

    if os.path.exists(os.path.join(basePath, relativePath)):
        path = os.path.join(basePath, relativePath)
        return path
    elif not os.path.exists(os.path.join(basePath, relativePath)):
        if os.path.exists(os.path.join(os.path.abspath("."), relativePath)):
            basePath = os.path.abspath(".")
        elif os.path.exists(os.path.join(os.path.abspath(".."), relativePath)):
            basePath = os.path.abspath("..")
        path = os.path.join(basePath, relativePath)
        return path


def print_version():
    """
    Simple log the version number of the multibootusb application
    :return:
    """
    log('multibootusb version: ' + mbusb_version())


def quote(text):
    """
    Function to quote the input word or sentence.
    :param text:    Any word or sentence.
    :return:        Quoted text or sentence. If already quoted the same text is returned.
    """
    if not is_quoted(text):
        return '"' + text + '"'
    else:
        return text


def is_quoted(text):
    """
    Function to check if word is quoted.
    :param text:    Any word or sentence with or without quote.
    :return:        True if text is quoted else False.
    """
    return bool(text.startswith("\"") and text.endswith("\""))


def has_digit(word):
    """
    Useful function to detect if input word contain digit.
    :param word:    Any alphanumeric word.
    :return:        True if the word has a digit else False.
    """
    return any(char.isdigit() for char in word)


def sys_64bits():
    """
    Detect if the host system is 64 bit.
    :return:    True if system is 64 bit.
    """
    return sys.maxsize > 2**32


def mbusb_log_file():
    """
    Function to genrate path to log file.
    Under linux path is created as /tmp/multibootusb.log
    Under Windows the file is created under installation directory
    """
    if platform.system() == "Linux":
        # home_dir = os.path.expanduser('~')
        # log_file = os.path.join(home_dir, "multibootusb.log")
        log_file = os.path.join(tempfile.gettempdir(), "multibootusb.log")
    elif platform.system() == "Windows":
        # log_file = os.path.join(tempfile.gettempdir(), "multibootusb", "multibootusb.log")
        log_file = os.path.join("multibootusb.log")

    return log_file


def multibootusb_host_dir():
    """
    Cross platform way to detect multibootusb directory on host system.
    :return: Path to multibootusb directory of host system.
    """
    if platform.system() == "Linux":
        home_dir = os.path.expanduser('~')
        mbusb_dir = os.path.join(home_dir, ".multibootusb")
    elif platform.system() == "Windows":
        mbusb_dir = os.path.join(tempfile.gettempdir(), "multibootusb")

    return mbusb_dir


def iso_cfg_ext_dir():
    """
    Function to return the path to ISO configuration file extraction directory.
    :return:    Path to directory where ISO config files will be extracted.
    """
    return os.path.join(multibootusb_host_dir(), 'iso_cfg_ext_dir')


def clean_iso_cfg_ext_dir(iso_cfg_ext_dir):
    """
    Clean old ISO config files extracted by previous use of multibootusb.
    :param iso_cfg_ext_dir: Path to config extract directory.
    :return:
    """
    if os.path.exists(iso_cfg_ext_dir):
        filelist = [f for f in os.listdir(iso_cfg_ext_dir)]
        for f in filelist:
            if os.path.isdir(os.path.join(iso_cfg_ext_dir, f)):
                shutil.rmtree(os.path.join(iso_cfg_ext_dir, f))
            else:
                os.remove(os.path.join(iso_cfg_ext_dir, f))
    else:
        log('iso_cfg_ext_dir directory does not exist.')


def copy_mbusb_dir_usb(usb_disk):
    """
    Copy the multibootusb directory to USB mount path.
    :param usb_mount_path: Path to USB mount.
    :return:
    """
#     from .iso import iso_size
    from .usb import details

    usb_details = details(usb_disk)
    usb_mount_path = usb_details['mount_point']
    result = ''
    if not os.path.exists(os.path.join(usb_mount_path, "multibootusb")):
        try:
            log('Copying multibootusb directory to ' + usb_mount_path)
            shutil.copytree(resource_path(os.path.join("data", "multibootusb")), os.path.join(usb_mount_path, "multibootusb"))

            result = True
        except:
            log('multibootusb directory could not be copied to ' + usb_mount_path)
            result = False
    else:
        log('multibootusb directory already exists. Not copying.')

    if not os.path.exists(os.path.join(usb_mount_path, 'EFI', 'BOOT', 'multibootusb_grub2.txt')):
        if not os.path.exists(os.path.join(usb_mount_path, 'EFI', 'BOOT')):
            log('EFI/BOOT directory does not exist. Creating new.')
            os.makedirs(os.path.join(usb_mount_path, 'EFI', 'BOOT'), exist_ok=True)
        if os.path.exists(os.path.join(usb_mount_path, 'EFI')):
            shutil.rmtree(os.path.join(usb_mount_path, 'EFI'))
        try:
            log('Copying EFI directory to ' + usb_mount_path)
            shutil.copytree(resource_path(os.path.join("data", "EFI")), os.path.join(usb_mount_path, "EFI"))
            result = True
        except Exception as e:
            log(e)
            result = False
    else:
        log('EFI directory already exist. Not copying.')

    # For backward compatibility
    if not os.path.exists(os.path.join(usb_mount_path, 'EFI', 'BOOT', 'bootx64-gpt.efi')):
        shutil.copy(resource_path(os.path.join('data', 'EFI', 'BOOT', 'bootx64-gpt.efi')),
                    os.path.join(usb_mount_path, 'EFI', 'BOOT', 'bootx64-gpt.efi'))

    if not os.path.exists(os.path.join(usb_mount_path, 'EFI', 'BOOT', 'bootx64-msdos.efi')):
        shutil.copy(resource_path(os.path.join('data', 'EFI', 'BOOT', 'bootx64-msdos.efi')),
                    os.path.join(usb_mount_path, 'EFI', 'BOOT', 'bootx64-msdos.efi'))

    if not os.path.exists(os.path.join(usb_mount_path, 'multibootusb', 'grub', 'core-gpt.img')):
        shutil.copy(resource_path(os.path.join('data', 'multibootusb', 'grub', 'core-gpt.img')),
                    os.path.join(usb_mount_path, 'multibootusb', 'grub', 'core-gpt.img'))

    if not os.path.exists(os.path.join(usb_mount_path, 'multibootusb', 'grub', 'core-msdos.img')):
        shutil.copy(resource_path(os.path.join('data', 'multibootusb', 'grub', 'core-msdos.img')),
                    os.path.join(usb_mount_path, 'multibootusb', 'grub', 'core-msdos.img'))

    return result


def read_input_yes():
    """
    List option and read user input
    :return: True if user selected yes or else false
    """
    yes_list = ['Y', 'y', 'Yes', 'yes', 'YES']
    no_list = ['N', 'n', 'No', 'no', 'NO']
    response = input("Please enter the option listed above : ")
    if response in yes_list:
        return True
    elif response in no_list:
        return False


def strings(filename, _min=4):
    with open(filename, errors="ignore") as f:
        result = ""
        for c in f.read():
            if c in string.printable:
                result += c
                continue
            if len(result) >= _min:
                yield result
            result = ""
        if len(result) >= _min:  # catch result at EOF
            yield result


def size_not_enough(iso_link, usb_disk):
    from .iso import iso_size
    from .usb import details
    isoSize = iso_size(iso_link)
    usb_details = details(usb_disk)
    usb_size = usb_details['size_free']

    return bool(isoSize > usb_size)


def mbusb_version():
    version = open(resource_path(os.path.join("data", "version.txt")), 'r').read().strip()
    return version


def check_text_in_file(file_path, text):
    """
    Helper function to check if a text exist in a file.
    :param file_path: Path to file
    :param text: Text to be searched
    :return: True if found else False
    """
    if not os.path.exists(file_path):
        return False
    else:
        with open(file_path) as data_file:
            return any(text in line for line in data_file)


def prepare_mbusb_host_dir():
    """
    Prepare multibootusb host directory and extract data files for use.
    :return:
    """
    home = multibootusb_host_dir()
    if not os.path.exists(home):
        os.makedirs(home)
    else:
        log("Cleaning old multibootusb directory...")
        clean_iso_cfg_ext_dir(os.path.join(home, "iso_cfg_ext_dir"))
        #shutil.rmtree(home)
        #os.makedirs(home)

    if not os.path.exists(os.path.join(home, "preference")):
        os.makedirs(os.path.join(home, "preference"))

    if not os.path.exists(os.path.join(home, "iso_cfg_ext_dir")):
        os.makedirs(os.path.join(home, "iso_cfg_ext_dir"))

    if os.path.exists(os.path.join(home, "syslinux", "bin", "syslinux4")):
        log("Syslinux exist in multibootusb directory...")
    else:
        log("Extracting syslinux to multibootusb directory...")
        if platform.system() == "Linux":
            if sys_64bits() is True:
                log('Host OS is 64 bit...')
                log("Extracting syslinux 64 bit...")
                # log(resource_path(os.path.join("data", "tools", "syslinux", "syslinux_linux_64.zip")))
                with zipfile.ZipFile(resource_path(os.path.join("data", "tools", "syslinux", "syslinux_linux_64.zip")), "r") as z:
                    z.extractall(home)
            else:
                log("Extracting syslinux 32 bit...")
                with zipfile.ZipFile(resource_path(os.path.join("data", "tools", "syslinux", "syslinux_linux.zip")), "r") as z:
                    z.extractall(home)
        else:
            with zipfile.ZipFile(resource_path(os.path.join("data", "tools", "syslinux", "syslinux_windows.zip")), "r") as z:
                z.extractall(home)
        log("Extracting syslinux modules to multibootusb directory...")
        with zipfile.ZipFile(resource_path(os.path.join("data", "tools", "syslinux", "syslinux_modules.zip")), "r") as z:
                z.extractall(os.path.join(home, "syslinux"))

    if os.listdir(os.path.join(home, "iso_cfg_ext_dir")):
        log(os.listdir(os.path.join(home, "iso_cfg_ext_dir")))
        log("iso extract directory is not empty.")
        log("Removing junk files...")
        for files in os.listdir(os.path.join(home, "iso_cfg_ext_dir")):
            if os.path.isdir(os.path.join(os.path.join(home, "iso_cfg_ext_dir", files))):
                log(os.path.join(os.path.join(home, "iso_cfg_ext_dir", files)))
                os.chmod(os.path.join(os.path.join(home, "iso_cfg_ext_dir", files)), 0o777)
                shutil.rmtree(os.path.join(os.path.join(home, "iso_cfg_ext_dir", files)))
            else:
                try:
                    log(os.path.join(os.path.join(home, "iso_cfg_ext_dir", files)))
                    os.chmod(os.path.join(os.path.join(home, "iso_cfg_ext_dir", files)), 0o777)
                    os.unlink(os.path.join(os.path.join(home, "iso_cfg_ext_dir", files)))
                    os.remove(os.path.join(os.path.join(home, "iso_cfg_ext_dir", files)))
                except OSError:
                    log("Can't remove the file. Skipping it.")


def grub_efi_exist(grub_efi_path):
    """
    Detect efi present in USB disk is copied by multibootusb.
    :param isolinux_path: Path to "grub efi image"
    :return: True if yes else False
    """
    from . import iso
    if grub_efi_path is not None:
        sl = list(iso.strings(grub_efi_path))
        for strin in sl:
            if re.search(r'multibootusb', strin, re.I):
                return True
        return False


def process_exist(process_name):
    """
    Detect if process exist/ running and kill it.
    :param process_name: process name to check
    :return: True if processis killed else False
    """
    if platform.system() == 'Windows':
        import signal
        import wmi
        c = wmi.WMI()
        for process in c.Win32_Process():
            if process_name in process.Name:
                log(process_name + ' exist...')
                log(str(process.ProcessId) + ' ' + str(process.Name))
                log("Having Windows explorer won't allow dd.exe to write ISO image properly."
                      "\nKilling the process..")
                try:
                    os.kill(process.ProcessId, signal.SIGTERM)
                    return True
                except:
                    log('Unable to kill process ' + str(process.ProcessId))

    return False


def write_to_file(filepath, text):
    """
    Simple function to write a text file
    :param filepath: Path to file
    :param text: Text to be written on to file
    :return:
    """
    with open(filepath, 'w') as f:
        f.write(text.strip())


class MemoryCheck():
    """
    Cross platform way to checks memory of a given system. Works on Linux and Windows.
    psutil is a good option to get memory info. But version 5.0 and only will work.
    Source: https://doeidoei.wordpress.com/2009/03/22/python-tip-3-checking-available-ram-with-python/
    Call this class like this: 
    mem_info = memoryCheck()
    print(mem_info.value)
    """

    def __init__(self):

        if platform.system() == 'Linux':
            self.value = self.linuxRam()
        elif platform.system() == 'Windows':
            self.value = self.windowsRam()
        else:
            log("MemoryCheck Class will only work on Linux and Windows.")

    def windowsRam(self):
        """
        Uses Windows API to check RAM
        """
        kernel32 = ctypes.windll.kernel32
        c_ulong = ctypes.c_ulong

        class MEMORYSTATUS(ctypes.Structure):
            _fields_ = [
                ("dwLength", c_ulong),
                ("dwMemoryLoad", c_ulong),
                ("dwTotalPhys", c_ulong),
                ("dwAvailPhys", c_ulong),
                ("dwTotalPageFile", c_ulong),
                ("dwAvailPageFile", c_ulong),
                ("dwTotalVirtual", c_ulong),
                ("dwAvailVirtual", c_ulong)
            ]

        memoryStatus = MEMORYSTATUS()
        memoryStatus.dwLength = ctypes.sizeof(MEMORYSTATUS)
        kernel32.GlobalMemoryStatus(ctypes.byref(memoryStatus))

        return int(memoryStatus.dwTotalPhys / 1024 ** 2)

    def linuxRam(self):
        """
        Returns the RAM of a Linux system
        """
        totalMemory = os.popen("free -m").readlines()[1].split()[1]
        return int(totalMemory)

if __name__ == '__main__':
    log(quote("""Test-string"""))
    log(has_digit("test-string-with-01-digit"))
    log(sys_64bits())
    log(multibootusb_host_dir())
    log(iso_cfg_ext_dir())
    strings_test = strings('../../text-stings.bin')
    log(strings_test)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Name:     mbusb_cli.py
# Purpose:  Module to handle command line options of multibootusb
# Authors:  Sundar
# Licence:  This file is a part of multibootusb package. You can redistribute it or modify
# under the terms of GNU General Public License, v.2 or above

import os
import ctypes
from . import usb
from . import gen
from .iso import *
from .uninstall_distro import *
from .distro import *
from .syslinux import *
from .install import *
from . import imager
from . import syslinux


def read_input_uninstall():
    response = False
    try:
        response = int(input("Please enter the number against the distro you need to uninstall: "))
    except ValueError:
        log('\nPlease provide valid integer from the above list.\n')

    return response


def is_root():
    """
    Check if user has admin rights
    :return: 
    """
    if platform.system() == 'Linux':
        if os.getuid() == 0:
            return True
    elif platform.system() == 'Windows':
        if ctypes.windll.shell32.IsUserAnAdmin() == 1:
            return True

    return False

def cli_install_distro():
#     if platform.system() == 'Linux':
#         if os.getuid() != 0:
#             exit("You need to have root privileges to run this script.\nPlease try again using 'sudo'. Exiting.")
#     elif platform.system() == 'Windows':
#
#         if admin.isUserAdmin():
#             admin.elevate()

    log('Starting multibootusb from Command line...')
    if usb.is_block(config.usb_disk) is False:
        log(config.usb_disk + ' is not a valid device partition...')
        exit(1)
    #elif integrity(config.image_path) is not True:
    #    log(config.image_path + ' failed to pass integrity check...')
    #    exit(1)
    else:
        usb_details = details(config.usb_disk)
        config.usb_mount = usb_details['mount_point']
        config.usb_uuid = usb_details['uuid']
        config.usb_label = usb_details['label']
        prepare_mbusb_host_dir()
        if isinstance(config.image_path, str) is True:
            iso_install(config.image_path)
        elif isinstance(config.image_path, list) is True:
            # Transfer the list to other variable and loop through iso image
            iso_list = config.image_path
            for config.image_path in iso_list:
                iso_install(config.image_path)


def iso_install(iso_image):
    """
    Script for installing iso image to a disk. This can be called by other script for auto install of many distros
    :param iso_image: Path to ISO image
    :return:
    """
    if size_not_enough(iso_image, config.usb_disk) is True:
        log(config.usb_disk + ' does not have enough space...')
    else:
        clean_iso_cfg_ext_dir(os.path.join(multibootusb_host_dir(), "iso_cfg_ext_dir"))  # Need to be cleaned everytime
        extract_cfg_file(iso_image)
        _distro = distro(iso_cfg_ext_dir(), iso_image)
        if _distro is not None:
            log('Initiating installation process for ' + iso.iso_basename(iso_image))
            log('Detected distro type is    :' + _distro)
            log('\nSelected ISO is          :' + quote(iso_name(iso_image)))
            log('Selected target device is  :' + quote(config.usb_disk), '\n')
            if config.yes is not True:
                log('Please confirm the option.')
                log('Y/y/Yes/yes/YES or N/n/No/no/NO')
                if read_input_yes() is True:
                    config.distro = _distro
                    copy_mbusb_dir_usb(config.usb_disk)
                    install_progress()
                    syslinux_distro_dir(config.usb_disk, iso_image, _distro)
                    syslinux_default(config.usb_disk)
                    replace_grub_binary()
                    update_distro_cfg_files(iso_image, config.usb_disk, _distro)
                    log('Finished installing ' + iso.iso_basename(iso_image))
                else:
                    log('Not proceeding. User cancelled the operation.')
            else:
                log('Skipping user confirmation for ' + iso_image)
                config.distro = _distro
                copy_mbusb_dir_usb(config.usb_disk)
                install_progress()
                syslinux_distro_dir(config.usb_disk, iso_image, _distro)
                syslinux_default(config.usb_disk)
                replace_grub_binary()
                update_distro_cfg_files(iso_image, config.usb_disk, _distro)
                log('Finished installing ' + iso.iso_basename(iso_image))
        else:
            log('\n\nSorry ' + iso_name(iso_image) + ' is not supported at the moment.\n'
                'Please report tissue at https://github.com/mbusb/multibootusb/issues\n')


def cli_uninstall_distro():
    distro_list = install_distro_list()
    if distro_list is not None:
        for index, _distro_dir in enumerate(distro_list):
            log(str(index) + '  --->>  ' + _distro_dir)
        user_input = read_input_uninstall()
        if user_input is not False:
            for index, _distro_dir in enumerate(distro_list):
                if index == user_input:
                    config.uninstall_distro_dir_name = _distro_dir
                    unin_distro()
    else:
        log('No distro installed on ' + config.usb_disk)


def cli_dd():
    """
    Function to write ISO image directly to USB disk using dd
    :return: 
    """
    if platform.system() == 'Linux':
        if config.usb_disk[-1].isdigit() is True:
            log('Selected USB is a disk partition. Please select the whole disk eg. \'/dev/sdb\'')
            sys.exit(2)
        elif is_root() is False:
            log("You need to have root privileges to run this script.\nPlease try again using admin privilege (sudo).")
            sys.exit(2)

    if not os.path.exists(config.image_path):
        log('ISO image path does not exist. Please correct the path.')
        sys.exit(2)
    else:
        if config.yes is not True:
            log('Initiating destructive writing process for ' + iso.iso_basename(config.image_path))
            log('\nSelected ISO is          :' + quote(iso_name(config.image_path)))
            log('Selected target device is  :' + quote(config.usb_disk))
            log('Writing ISO directly to target USB disk ' + quote(config.usb_disk) + ' will DESTROY ALL DATA.' + '\n')
            log('Please confirm the option.')
            log('Y/y/Yes/yes/YES or N/n/No/no/NO')
            if read_input_yes() is True:
                if platform.system() == 'Linux':
                        imager.dd_linux()
                else:
                    imager.dd_win()
            else:
                log('Operation cancelled by user. Exiting...')
                sys.exit(2)
        else:
            log('\nAuto install is not recommended in direct writing method. Please choose without \'-y\' option.\n')
            sys.exit(2)


def cli_install_syslinux():
    """
    Install syslinux on a target USB disk. It will installed on 'multibootusb' directory
    :return: 
    """
    if platform.system() == 'Linux':
        if config.usb_disk[-1].isdigit() is not True:
            log('Selected USB disk is not a partition. Please enter the partition eg. \'/dev/sdb1\'')
            sys.exit(2)
        elif is_root() is False:
            log("You need to have root privileges to run this script.\nPlease try again using admin privilege (sudo).")
            sys.exit(2)

    if config.yes is not True:
        log('\nInitiating process for installing syslinux on ' + config.usb_disk)
        log('Selected target device is  : ' + quote(config.usb_disk))
        log('Syslinux install directory :  \'multibootusb\'\n')
        log('Please confirm the option.')
        log('Y/y/Yes/yes/YES or N/n/No/no/NO')
        if read_input_yes() is True:
            if syslinux.syslinux_default(config.usb_disk) is True:
                log('Syslinux successfully installed on ' + config.usb_disk)
            else:
                log('Failed to install syslinux on ' + config.usb_disk)
        else:
            log('Operation cancelled by user. Exiting...')
            sys.exit(2)
    else:
        log('\nSkipping user input and installing syslinux on ' + config.usb_disk)
        if syslinux.syslinux_default(config.usb_disk) is True:
            log('Syslinux successfully installed on ' + config.usb_disk)
        else:
            log('Failed to install syslinux on ' + config.usb_disk)
        sys.exit(2)




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Name:     uninstall.py
# Purpose:  Script to uninstall multibootusb from source, previously installed by install.py
# Authors:  Sundar
# Licence:  This file is a part of multibootusb package. You can redistribute it or modify
# under the terms of GNU General Public License, v.2 or above

import os
import sys
import shutil

if not os.getuid() == 0:
    print("You must run this file with admin privilege.")
    print("Try sudo ./uninstall.py")
    sys.exit(0)
else:
    if os.path.exists("./.install_files.txt"):
        with open("./.install_files.txt", "r") as f:
            file_list = f.readlines()
        for f in file_list:
            print("Removing " + f.replace('\n', ''))
            if os.path.isfile(f.replace('\n', '')):
                os.remove(f.replace('\n', ''))
            elif os.path.isdir(f.replace('\n', '')):
                shutil.rmtree(f.replace('\n', ''))
        if os.path.exists('/usr/share/multibootusb'):
            shutil.rmtree('/usr/share/multibootusb')

        print("multibootusb is successfully unistalled...")
    else:
        print("Unable to find install file list.")
        print("This script works only if you have installed multibootusb using install.py script.")



#!/usr/bin/env python3
# -*- coding: utf-8 -*
# Name:     isodump3.py
# Purpose:  Module to list and extract iso files.
# Authors:  LiQiong Lee (written exclusively for multibootusb)
# Licence:  This file is a part of multibootusb package. You can redistribute it or modify
# under the terms of GNU General Public License version 3
# Credit :  I am grateful to LiQiong Lee. He not only wrote this module for multibootusb, but also extended the same
#           to python3 within short time after request.

""" ISO9660fs
Dump raw meta data of iso9660 file system.
Extract directories and files.
"""
##
## Extract directory or file from iso.
## Support RRIP.
##

# Author  : joni <joni.kartore.lee@gmail.com>
# version : 1.0

import sys
import struct
import os
import re
import stat
from ctypes import *
from . import config
from . import gen


BLOCK_SIZE = 2048
S_IFSOCKET = 0o140000
S_IFLINK   = 0o120000
S_IFREG    = 0o100000
S_IFBLK    = 0o060000
S_IFCHR    = 0o020000
S_IFDIR    = 0o040000
S_IFIFO    = 0o010000

E_SUCCESS = 0
E_FAILURE = -1
E_DEVICEFILE = -2  # can't write device file

class PrimaryVolume(Structure):
    def  __init__(self):
        self.sysIdentifier = ""
        self.volIdentifier = ""
        self.volSize   = 0
        self.volSeq    = 0
        self.blockSize = 0
        self.ptSize    = 0
        self.ptLRd     = 0
        self.fsVer     = 0
        self.rootLoc   = 0
        self.rootTotal = 0

class Rrip(Structure):
    def __init__(self):
        self.offset  = -1
        self.altname = ""
        self.devH    =  0
        self.devL    =  0
        self.fMode   =  0

class DirRecord(Structure):
    def __init__(self):
       self.lenDr    =    0
       self.lenEattr =    0
       self.locExtent=    0
       self.lenData  =    0
       self.dtYear   =    0
       self.dtMonth  =    0
       self.dtHour   =    0
       self.dtMinute =    0
       self.dtSecond =    0
       self.dtOffset =    0
       self.fFlag    =    0
       self.fUnitSize=    0
       self.gapSize  =    0
       self.volSeqNr =    0
       self.lenFi    =    0
       self.fIdentifier = ""
       self.sysUseStar = 0
       self.suspBuf =  ""
       self.rrip = None

class PathTabelItem(Structure):
    def __init__(self):
        self.lenDi       = 0
        self.lenEattr    = 0
        self.locExtenti  = 0
        self.pdirNr      = 0
        self.fIdentifier = ""

class ISO9660:
    """
    This class can dump iso9660 file system meta data and extract files.
    Support:
        RRIP extension.
    """

    def __init__(self, isofile):
        try:
            f = open(isofile, 'rb')
        except(IOError):
            sys.stderr.write("can't open {0}".format(isofile))
            raise

        if os.path.getsize(isofile) == 0:
            raise IOError("File {0} appears to be empty".format(isofile))

        self.isoFile = f
        self.priVol = None
        self.rootDir = None
        self.rripOffset = -1

        desc_nr = 0
        while True:
            desc_nr = desc_nr + 1
            try:
                self.isoFile.seek(BLOCK_SIZE*(15+desc_nr))
                volume_dsc = self.isoFile.read(BLOCK_SIZE)
                flag = struct.unpack('B',volume_dsc[0:1])[0]
                if flag == 1:
                    self.__readPrimaryVolume__(volume_dsc)
                    continue
                if flag == 255:
                    break
            except Exception as e:
                gen.log("Got exception when init iso file:", sys.exc_info()[0])
                self.priVol = None
                self.rootDir = None
                break

    def __del__(self):
        self.isoFile.close()

    def __readPrimaryVolume__(self, volume_dsc):
        """ Dump primary volume descriptor """
        global BLOCK_SIZE
        priVol = PrimaryVolume()
        priVol.sysIdentifier = volume_dsc[8:40]
        priVol.volIdentifier = volume_dsc[40:72]
        priVol.volSize = struct.unpack('<L',volume_dsc[80:84])[0]
        priVol.volSeq = struct.unpack('<H',volume_dsc[124:126])[0]
        priVol.blockSize = struct.unpack('<H',volume_dsc[128:130])[0]
        priVol.ptSize = struct.unpack('<L',volume_dsc[132:136])[0]
        priVol.ptLRd = struct.unpack('<L',volume_dsc[140:144])[0]
        priVol.fsVer = struct.unpack('B', volume_dsc[881:882])[0]
        dirRec = self.readDirrecord(volume_dsc[156:190])
        priVol.rootLoc = dirRec.locExtent
        priVol.rootTotal = dirRec.lenData
        BLOCK_SIZE = priVol.blockSize

        # Check RRIP
        #gen.log("loc extent(%d)"%(dirRec.locExtent))
        self.priVol = priVol # readDirItems will use self.priVol
        root_dir = self.readDirItems(dirRec.locExtent, priVol.rootTotal)[0]
        rripNode = self.__rripLoop__(root_dir.suspBuf, root_dir.lenDr-root_dir.sysUseStar)
        if rripNode.offset != -1:
            self.rripOffset = rripNode.offset
            #gen.log("RRIP: rrip_offset %d"%(self.rripOffset))
        else:
            gen.log("This ISO doesn't support RRIP")
        self.rootDir = root_dir

    #  Rrip extension
    def __rripLoop__(self, desc_buf, len_buf):

        if self.rripOffset > 0:
            entry_buf = desc_buf[self.rripOffset:]
            gen.log("__rripLoop__ offset:%d"%(self.rripOffset))
        else:
            entry_buf = desc_buf

        rr = Rrip()
        while True:
            ce_blk = 0
            ce_len = 0
            ce_off = 0
            head = 0
            len_entry = 0

            while True:
                #gen.log(("\n%d, %d\n")%(len_buf, head))
                head += len_entry
                if len_buf - head < 4: # less than one entry
                    break
                entry_buf = entry_buf[len_entry:]

                sig1 = struct.unpack("B", entry_buf[0:1])[0]
                sig2 = struct.unpack("B", entry_buf[1:2])[0]
                len_entry = struct.unpack("B", entry_buf[2:3])[0]
                ver = struct.unpack("B", entry_buf[3:4])[0]
                #if len_entry == 0:
                #    gen.log "Got a entry in __rripLoop__ (%c,%c) of SUSP with length:(%d),version:(%d)-->"%(sig1,sig2,len_entry, ver),
                if len_entry == 0:
                   break;

                if sig1 == ord('S') and sig2 == ord('P'):
                    ck1 = struct.unpack("B", entry_buf[4:5])[0]
                    ck2 = struct.unpack("B", entry_buf[5:6])[0]
                    skip = struct.unpack("B", entry_buf[6:7])[0]
                    #gen.log "-->(0x%x==0xBE,0x%x==EF,%d)" %(ck1, ck2, skip)
                    if ck1 == 0xBE and ck2 == 0xEF:
                        rr.offset = skip
                    continue

                if sig1 == ord('C') and sig2 == ord('E'):
                    ce_blk = struct.unpack("<L", entry_buf[4:8])[0]
                    ce_off = struct.unpack("<L", entry_buf[12:16])[0]
                    ce_len = struct.unpack("<L", entry_buf[20:24])[0]
                    #gen.log "-->(%d,%d,%d)" %(ce_blk, ce_off, ce_len)
                    continue

                if sig1 == ord('N') and sig2 == ord('M'):
                    flag = struct.unpack("B", entry_buf[4:5])[0]
                    #gen.log "-->(flag:(0x%x), name:(%s))" %(flag, entry_buf[5:len_entry])
                    if flag == 0x02:     # FLAG_CURRENT
                        rr.altname += "."
                    elif flag == 0x04:   # FLAG_PARENT
                        rr.altname += ".."
                    elif flag == 0x01 or flag ==0:  # 1:FLAG_CONTINUE
                        rr.altname += entry_buf[5:len_entry].decode()
                    continue

                if sig1 == ord('P') and sig2 == ord('N'):
                    rr.devH = struct.unpack("<L", entry_buf[4:8])[0]
                    rr.devL = struct.unpack("<L", entry_buf[12:16])[0]
                    continue

                if sig1 == ord('E') and sig2 == ord('R'):
                    len_id = struct.unpack("B", entry_buf[4:5])[0]
                    len_des = struct.unpack("B", entry_buf[5:6])[0]
                    len_src = struct.unpack("B", entry_buf[6:7])[0]
                    ext_ver = struct.unpack("B", entry_buf[7:8])[0]
                    continue

                if sig1 == ord('P') and sig2 == ord('X'):
                    rr.fMode = struct.unpack("<L", entry_buf[4:8])[0]
                    s_link = struct.unpack("<L", entry_buf[12:16])[0]
                    uid = struct.unpack("<L", entry_buf[20:24])[0]
                    gid = struct.unpack("<L", entry_buf[28:32])[0]
                    continue

                if sig1 == ord('S') and sig2 == ord('T'):
                    return rr

                #gen.log "\n"
            # while (True) end #

            if ce_len > 0:
                #gen.log " Read CE block, (%d, %d, %d)"%(ce_blk, ce_len, ce_off)
                self.isoFile.seek(ce_blk*BLOCK_SIZE + ce_off)
                entry_buf = self.isoFile.read(ce_len)
                len_buf = ce_len
            else:
                break
        # while (True) end #
        return rr

    def checkISOBootable(self):
        """ Struct of a classical generic MBR.

            0x0000      Bootstrap Code area
            -----------------------------------------
            0x01BE
            ..          Partition table
            0x01EE
            ------------------------------------------
            0x01FE         55h
                                       Boot signature
            0x01FF         AAh

        """
        self.isoFile.seek(0x01FE)
        h = self.isoFile.read(2)
        s1 = struct.unpack('B', h[0:1])[0]
        s2 = struct.unpack('B', h[1:2])[0]

        #gen.log "-->(0x%x,0x%x)" %(s1, s2)

        if (s1 == 0x55) and (s2 == 0xAA):
            result = True   # "Bootable"
        else:
            result = False  # "Not bootable"

        return result

    def searchDir(self, path):
        # /root/abc/ - ['', 'root', 'abc', '']
        # /root/abc  - ['', 'root', 'abc']
        # /          - ['', '']
        dircomps = path.split('/')
        if dircomps[-1] == '':
            dircomps.pop()
        if dircomps == []:
            return

        if self.priVol == None:
           return

        if len(dircomps) == 1:
            return self.rootDir

        pdir_loc = self.priVol.rootLoc
        pdir_len = self.priVol.rootTotal
        i_dircomp = 1

        while True:
            found = False
            dirs = self.readDirItems(pdir_loc, pdir_len)
            for item in dirs:
                if item.fIdentifier == dircomps[i_dircomp]:
                    pdir_loc = item.locExtent
                    pdir_len = item.lenData
                    found = True
                    #gen.log "found (%s)"%(dircomps[i_dircomp])
                    break
            if found: # advacne
                if i_dircomp < len(dircomps)-1:
                    i_dircomp = i_dircomp + 1
                else:
                    return item
            else:
                gen.log("can't find " + dircomps[i_dircomp])
                return None

    def readDirrecord(self, desc_buf):
        """ Dump file  dirctory record
        Return a directory record reading from a Directory Descriptors.
        """
        dirRec = DirRecord()
        try:
            dirRec.lenDr = struct.unpack("B", desc_buf[0:1])[0]
            if dirRec.lenDr == 0:
                return None
        except:
           return None

        dirRec.lenEattr = struct.unpack("B", desc_buf[1:2])[0]
        dirRec.locExtent = struct.unpack("<L", desc_buf[2:6])[0]
        dirRec.lenData = struct.unpack("<L", desc_buf[10:14])[0]
        dirRec.fFlag = struct.unpack("B", desc_buf[25:26])[0]
        dirRec.fUnitSize = struct.unpack("B", desc_buf[26:27])[0]
        dirRec.gapSize = struct.unpack("B", desc_buf[27:28])[0]
        dirRec.volSeqNr = struct.unpack("<H", desc_buf[28:30])[0]
        dirRec.lenFi = struct.unpack("B", desc_buf[32:33])[0]
        dirRec.fIdentifier = ""
        if dirRec.lenFi == 1:
            dirRec.fIdentifier  = struct.unpack("B", desc_buf[33:34])[0]
            if dirRec.fIdentifier  == 0:
                dirRec.fIdentifier = "."
            elif dirRec.fIdentifier == 1:
                dirRec.fIdentifier = ".."
        else:
            dirRec.fIdentifier = desc_buf[33:33+dirRec.lenFi].decode()
            idx = dirRec.fIdentifier.rfind(";")
            if idx != -1:
                dirRec.fIdentifier = dirRec.fIdentifier[0:idx]

        dirRec.suspBuf = ""
        dirRec.sysUseStar = 34 + dirRec.lenFi -1
        if dirRec.lenFi % 2 == 0:
            dirRec.sysUseStar += 1

        # Extension Attribute
        if dirRec.lenDr > dirRec.sysUseStar+4:
            if dirRec.locExtent == self.priVol.rootLoc:
                dirRec.suspBuf = desc_buf[dirRec.sysUseStar:dirRec.lenDr]
            suspBuf = desc_buf[dirRec.sysUseStar:dirRec.lenDr]
            if self.rripOffset != -1:
                rripNode = self.__rripLoop__(suspBuf, dirRec.lenDr-dirRec.sysUseStar)
                dirRec.rrip = rripNode
                if rripNode != None:
                    if rripNode.altname != "":
                        dirRec.fIdentifier = rripNode.altname
                        dirRec.lenFi = len(rripNode.altname)
                        #gen.log "rrip_altname: %s"%(dirRec.fIdentifier)
                # if rripNode end #
            # if self.rripOffset != -1 end #
        # if dirRec.lenDr > .. end #
        return dirRec

    def readDirItems(self, block_nr=None, total=None):
        """ Read file dirctory records
         Read dirctory records from 'block_nr' with a length of 'total'.
         Return a list containing directory records(DirRecord).
        """
        dirs = []
        total_blk = (total+BLOCK_SIZE-1)//BLOCK_SIZE
        i_blk = 0
        while i_blk < total_blk:
            self.isoFile.seek((block_nr+i_blk)*BLOCK_SIZE)
            desc_buf = self.isoFile.read(BLOCK_SIZE)
            i_blk = i_blk + 1
            while True:
                dirItem = self.readDirrecord(desc_buf)
                if dirItem == None:
                    break

                dirs.append(dirItem)
                if desc_buf.__len__() > dirItem.lenDr:
                    desc_buf = desc_buf[dirItem.lenDr:]
                else:
                   break
        return dirs

    def readPathtableL(self):
        """ Read path table of L typde """
        if self.priVol == None:
           return
        block_nr = self.priVol.ptLRd
        total = self.priVol.ptSize

        path_table = []
        self.isoFile.seek(block_nr*BLOCK_SIZE)
        ptbuf = self.isoFile.read((BLOCK_SIZE * ((total+BLOCK_SIZE-1)//BLOCK_SIZE)))
        i = 0
        r_size = 0
        while True :
            i = i+1
            t = PathTabelItem()

            t.lenDi = struct.unpack('B', ptbuf[0:1])[0]
            t.lenEattr = struct.unpack('B', ptbuf[1:2])[0]
            t.locExtent = struct.unpack('<L', ptbuf[2:6])[0]
            t.pdirNr = struct.unpack('<H', ptbuf[6:8])[0]
            t.fIdentifier = ptbuf[8:8+t.lenDi].decode()
            path_table.append(t)
            if t.lenDi % 2 :
                len_pd = 1
            else:
                len_pd = 0

            r_size += 9+t.lenDi-1+len_pd
            if r_size >= total:
                break
            ptbuf = ptbuf[9+t.lenDi-1+len_pd:]
        # while True
        return path_table

    # @path -- path within iso file system.
    # @output -- what local path you want write to.
    # @pattern -- regular expression.
    # @r -- recursion flag, write the whole sub-directories or not.
    # @all_type -- which file type should be writed.
    #              False: Write regular type files only.
    #              True: Wirte all types files (regular, device file, link, socket, etc)
    def writeDir(self, path, output, pattern="", r=True, all_type=False):
        """ Extract a directory
        Return 0 means success otherwise failure.
        """
        d = self.searchDir(path)
        if d != None:
            if output.endswith("/"):
                output = output[0:-1]
            # Try to make target directory.
            if not os.path.exists(output):
                try:
                    os.makedirs(output)
                except(OSError):
                    sys.stderr.write("can't make dirs({0})\n".format(output))
                    return E_FAILURE
            pp = None
            if pattern != "":
                p = r'{0}'.format(pattern)
                pp = re.compile(p)
            #gen.log "writeDir: flag(%x)"%(d.fFlag)
            if d.fFlag & 0x02 == 0x02:
                # Check if a clean directory.
                #try:
                #    if len(os.listdir(output)) > 0:
                #        sys.stderr.write("The target directory is not empty\n")
                #        return E_FAILURE
                #except(OSError):
                #    sys.stderr.write("can't access dirs({0})\n".format(p))
                #    return E_FAILURE
                self.writeDir_r(output, d, pp, r, all_type)
                return E_SUCCESS
            else:
                return self.writeFile(d, output+path, all_type)
        else:
            return E_FAILURE

    def writeDir_r(self, det_dir, dire, pp, r, all_type):
        #gen.log "writeDir_r:(%s)"%(det_dir)
        dirs = self.readDirItems(dire.locExtent, dire.lenData)
        for d in dirs:
            if not d.fIdentifier in [".", ".."]:
                if (pp != None) and (pp.search(d.fIdentifier) == None):
                    match = False
                else:
                    match = True
                #gen.log "mathing %s, %s, (%x)"%(match, d.fIdentifier, d.fFlag)
                p = det_dir + "/" + d.fIdentifier
                if d.fFlag & 0x02 == 0x02:
                    if not os.path.exists(p):
                        os.makedirs(p, 0o777)
                    if r:
                        if match:
                            self.writeDir_r(p, d, None, r, all_type) # Don't need to match subdirectory.
                        else:
                            self.writeDir_r(p, d, pp, r, all_type)
                elif match:
                    self.writeFile(d, p, all_type)
            # if not d.fIdentifier end #
        # for d in dirs end #

    def writeFile(self, dirRec, detFile, all_type):
        """ Write a file to detFile
        Return 0 means success otherwise failure.
        """
        global file_out
        if detFile == "" or dirRec == None:
            sys.stderr.write("can't write file\n")
            return E_FAILURE

        #gen.log "write file (%s)"%(detFile)
        config.status_text = detFile

        dirname = os.path.dirname(detFile)
        if not os.path.exists(dirname):
            try:
                os.makedirs(dirname, 0o777)
            except(OSError):
                sys.stderr.write("can't makedirs\n")
                return E_FAILURE

        if all_type == True:
            # device file
            if dirRec.rrip != None and (dirRec.rrip.devH != 0 or dirRec.rrip.devL != 0):
                #fFlag == 0
                high = dirRec.rrip.devH
                low = dirRec.rrip.devL
                if high == 0:
                    device = os.makedev(os.major(low), os.minor(low))
                else:
                    device = os.makedev(high, os.minor(low))
                try:
                    mode = dirRec.rrip.fMode & 0o770000
                    if mode == S_IFCHR:
                        os.mknod(detFile, 0o777|stat.S_IFCHR, device)
                    elif mode  == S_IFBLK:
                        os.mknod(detFile, 0o777|stat.S_IFBLK, device)
                except(OSError):
                    sys.stderr.write("can't mknode, maybe no permission\n")
                    return E_DEVICEFILE

                return E_SUCCESS

        loc = dirRec.locExtent
        length = dirRec.lenData
        self.isoFile.seek(BLOCK_SIZE * loc)
        #gen.log "file length(%d)"%(length)
        r_size = BLOCK_SIZE*1024*50 #100M cache

        try:
            f_output = open(detFile, 'wb', r_size)
        except(IOError):
            sys.stderr.write("can't open{0} for write\n".format(detFile))
            return E_FAILURE

        while True:
            if length == 0:
                break
            elif length <= r_size:
                r_size = length
                length = 0
            else:
                length = length - r_size

            buf = self.isoFile.read(r_size)
            f_output.write(buf)
            f_output.flush()
        # while True end.
        f_output.close()
        return E_SUCCESS

    def readDir(self, dir_path, r=True):
        file_list = []
        d = self.searchDir(dir_path)
        if d != None:
            if (d.fFlag & 0x02) == 0x02:
                #gen.log "readDir (%x, %x)"%(d.locExtent, d.lenData)
                if dir_path.endswith("/"):
                    dir_path = dir_path[0:-1]
                self.readDir_r(file_list, dir_path, d, r)
            # if (d.fFlag & 0x02) == 0x02: #
        # if d != None:
        return file_list

    def readDir_r(self, file_list, dir_path, dire, r):
        if (dire.fFlag & 0x02) != 0x02:
            return
        dirs = self.readDirItems(dire.locExtent, dire.lenData)
        for d in dirs:
            if not d.fIdentifier in [".", ".."]:
                p = dir_path + "/" + d.fIdentifier
                file_list.append(p)
                if r:
                    self.readDir_r(file_list, p, d, r)
            # if not d.fIdentifier #
        # for d in dirs: #

    def checkIntegrity(self):
        if self.priVol == None: # no primary volume
            return False

        if self.priVol.ptSize == 0: # empty ?
            return True

        path_table = self.readPathtableL()
        if path_table == []: # pathtable record is broken.
            return False

        # find last file item to check
        for dr in reversed(path_table):
            #gen.log dr.fIdentifier
            dirs = self.readDirItems(dr.locExtent, BLOCK_SIZE)
            if len(dirs) > 2:
                dot = dirs[0]
                dirs2 = self.readDirItems(dot.locExtent, dot.lenData) # get the whole items.
                for dr2 in reversed(dirs2): # search last file item.
                    if dr2.fFlag == 0:
                        #gen.log "get last file(%s)"%(dr2.fIdentifier)
                        try:
                            #self.isoFile.seek(BLOCK_SIZE * dr2.locExtent+dr2.lenData)
                            lastfile_end = BLOCK_SIZE * dr2.locExtent + dr2.lenData
                            self.isoFile.seek(0, os.SEEK_END)
                            iso_end = self.isoFile.tell()
                            #gen.log("%d-->%d")%(lastfile_end, iso_end)
                            if iso_end >= lastfile_end:
                                return True
                            else:
                                return False
                        except(IOError):
                            #gen.log "exception when seek. iso is broken"
                            return False
            elif len(dirs) < 2: # Dir record is broken. At least, should have two entries.
               return False
        return True

###########################################################################
def dump_dir_record(dirs):
    """ Dump all the file directory records contained in desc_buf """

    gen.log("Dump file/directory record")
    gen.log("===========================", end="\n")
    if dirs != None:
        for f in dirs:
            gen.log("length of directory record:(0x%x), length of extend attribute:(%d), \
location of record:(%d)BLOCK->(0x%x), data length(%d) size of file unit:(%d), \
interleave gap size:(%d), file flag:(0x%x),name length:(%d) identify:(%s)\n" \
%(f.lenDr, f.lenEattr, f.locExtent, f.locExtent*BLOCK_SIZE,f.lenData, \
  f.fUnitSize, f.gapSize, f.fFlag, f.lenFi, f.fIdentifier))

def dump_pathtable_L(path_table):
    """ Dump path table of L typde """

    gen.log("Dump path table")
    gen.log("================", end="\n")
    #path_table = readPathtableL()
    i = 0
    for t in path_table:
        i = i + 1
        if t.lenDi == 1:
            if t.fIdentifier in [0, 1]:
                gen.log("is a root directory(%d)" %(is_root))
        gen.log("%d->length of identify:(%d), length of extend attribute:(%d), \
local:(%d)->(0x%x), parent dir number:(%d), identify:(%s)\n" \
%(i, t.lenDi, t.lenEattr, t.locExtent, t.locExtent*BLOCK_SIZE, t.pdirNr, t.fIdentifier))

def dump_primary_volume(privol=None):
    """ Dump primary volume descriptor """

    if privol == None:
        gen.log("Can't dump, maybe iso is broken")
        return
    gen.log("===== Dump primary volume descriptor ==")

    gen.log("System Identifier:(%s)" %(privol.sysIdentifier.decode()))
    gen.log("Volume Identifier:(%s)" %privol.volIdentifier.decode())
    gen.log("Volume Space size:(0x%x)BLOCKS(2kB)" %privol.volSize)
    gen.log("Volume sequence number:(%d)" %(privol.volSeq))
    gen.log("logic block size:(0x%x)" %(privol.blockSize))
    gen.log("Volume path talbe L's BLOCK number is :(0x%x-->0x%x), size(%d)" %(privol.ptLRd, privol.ptLRd*BLOCK_SIZE, privol.ptSize))
#    gen.log "Abstract File Identifier: (%s)" %(volume_dsc[739:776])
#    gen.log "Bibliographic File Identifier: (%s)" %(volume_dsc[776:813])
    gen.log("pathtable locate (%d)" %(privol.ptLRd))
    gen.log("File Structure Version:(%d)" %(privol.fsVer))
    gen.log("Root directory is at (%d)block, have(0x%x)bytes" %(privol.rootLoc, privol.rootTotal))
#    dump_dir_record(None, 23, 1)

def dump_boot_record(volume_dsc):
    """ Dump boot record  """

    gen.log("===== Dump boot record ==")
    std_identifier = volume_dsc[1:6]
    gen.log("Standard Identifier:(%s)" %std_identifier)

    vol_ver = struct.unpack('B', volume_dsc[6])
    gen.log("Volume descriptor version:(%d)" %vol_ver)

    bootsys_identifier = volume_dsc[7:39]
    gen.log("boot system identifier(%s)" %bootsys_identifier)

    boot_identifier = volume_dsc[39:71]
    gen.log("boot identifier(%s)" %boot_identifier)

def usage():
    """ Prompt user how to use   """
    gen.log("""
Usage: isodump  dump-what [options]  iso-file
       [dump-what]
       -----------
       boot               - Dump boot record.
       primary-volume     - Dump primary volume.
       pathtable          - Dump path table.
       dir-record [block number] [length] - Dump a raw data of a Directory Record

       iso:/dir [-r]  [-o output] [-p pattern]  - Dump a dirctory or file to [output]
           -r    recursively visit directory.
           -p    spcify a Regular expression pattern for re.search(pattern,).

isodump xx.iso              - Dump the root directory
isodump pathtable xx.iso    - Dump the path table record.

isodump iso:/ -r     xx.iso
    -- Dump the root directory of xx.iso recursively.

isodump iso:/ -r -o /tmp/iso    xx.iso
    -- Extract the iso to /tmp/iso/.

isodump iso:/boot -o /tmp/iso/boot    xx.iso
    -- Extract the /boot directory of xx.iso to /tmp/iso/boot.

isodump iso:/boot/grup.cfg -o /tmp/grub.cfg  xx.iso
    -- Extract the file "grup.cfg" to "/tmp/grub.cfg"

isodump iso:/boot -r -o /tmp/iso -p "*.cfg"  xx.iso
    -- Extract any files or directories under /boot maching "*.cfg" to /tmp/iso/.
""")
    sys.exit(-1)

if __name__ == '__main__':
    argv = sys.argv
    if len(argv) < 3:
        usage()

    iso9660fs = ISO9660(argv[-1])
    integrity = iso9660fs.checkIntegrity()
    if integrity == False:
        gen.log("iso file is broken")
        sys.exit(-1)

    dump_what = argv[1]

    if dump_what == "primary-volume":
        dump_primary_volume(iso9660fs.priVol)
    elif dump_what == "pathtable":
        path_table = iso9660fs.readPathtableL()
        dump_pathtable_L(path_table)
    if dump_what == "dir-record":
        if len(argv) == 5:
            gen.log("dump dir-record (%s, %s)"%(argv[2], argv[3]))
            dirs = iso9660fs.readDirItems(int(argv[2]), int(argv[3]))
            dump_dir_record(dirs)
        else:
            usage()
    elif dump_what.startswith("iso:"):
        o_path = ""
        r = False
        o = False
        p = False
        pattern = ""
        for arg in argv[2:-1]:
            if arg == "-r":
                r = True
                o = False
                p = False
            elif arg == "-o":
                o = True
                p = False
            elif arg == "-p":
                o = False
                p = True
            elif o == True:
                o_path = arg
                o = False
            elif p == True:
                pattern = arg
                p = False

        isodir = dump_what[4:]
        if o_path == "":
            gen.log("dump_dir(%s)"%(isodir))
            filelist = iso9660fs.readDir(isodir, r)
            if filelist == []:
                gen.log("can't read any file from (%s)"%(isodir))
            else:
                for f in filelist:
                    gen.log(f)
        else:
            gen.log("writeDir(%s)->(%s) with pattern(%s)"%(isodir, o_path, pattern))
            sys.exit(iso9660fs.writeDir(isodir, o_path, pattern, r, True))
